@@ -10,7 +10,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "sg_include.h"
-#include "sg_err.h"
+#include "sg_lib.h"
 
 /* A utility program for the Linux OS SCSI generic ("sg") device driver.
 *  Copyright (C) 1999 D. Gilbert and P. Allworth
@@ -143,9 +143,9 @@ int read_capacity(int sg_fd, int * num_sect, int * sect_sz)
         return -1;
     }
     res = sg_err_category3(&io_hdr);
-    if (SG_ERR_CAT_MEDIA_CHANGED == res)
+    if (SG_LIB_CAT_MEDIA_CHANGED == res)
         return 2; /* probably have another go ... */
-    else if (SG_ERR_CAT_CLEAN != res) {
+    else if (SG_LIB_CAT_CLEAN != res) {
         sg_chk_n_print3("read capacity", &io_hdr);
         return -1;
     }
@@ -253,13 +253,13 @@ int sg_finish_io(Rq_coll * clp, int wr, Rq_elem ** repp)
         *repp = rep;
 
     switch (sg_err_category3(hp)) {
-        case SG_ERR_CAT_CLEAN:
+        case SG_LIB_CAT_CLEAN:
             break;
-        case SG_ERR_CAT_RECOVERED:
+        case SG_LIB_CAT_RECOVERED:
             printf("Recovered error on block=%d, num=%d\n",
                    rep->blk, rep->num_blks);
             break;
-        case SG_ERR_CAT_MEDIA_CHANGED:
+        case SG_LIB_CAT_MEDIA_CHANGED:
             return 1;
         default:
             sg_chk_n_print3(rep->wr ? "writing": "reading", hp);
@@ -295,33 +295,6 @@ int sz_reserve(int fd, int bs, int bpt)
     return 0;
 }
 
-int get_num(char * buf)
-{
-    int res, num;
-    char c, cc;
-
-    res = sscanf(buf, "%d%c", &num, &c);
-    if (0 == res)
-        return -1;
-    else if (1 == res)
-        return num;
-    else {
-        cc = (char)toupper(c);
-        if ('B' == cc)
-            return num * 512;
-        else if ('C' == cc)
-            return num;
-        else if ('K' == cc)
-            return num * 1024;
-        else if ('M' == cc)
-            return num * 1024 * 1024;
-        else {
-            printf("unrecognized multiplier\n");
-            return -1;
-        }
-    }
-}
-
 void init_elems(Rq_coll * clp)
 {
     Rq_elem * rep;
@@ -648,23 +621,23 @@ int main(int argc, char * argv[])
         else if (strcmp(key,"of") == 0)
             strcpy(outf, buf);
         else if (0 == strcmp(key,"ibs"))
-            ibs = get_num(buf);
+            ibs = sg_get_num(buf);
         else if (0 == strcmp(key,"obs"))
-            obs = get_num(buf);
+            obs = sg_get_num(buf);
         else if (0 == strcmp(key,"bs"))
-            rcoll.bs = get_num(buf);
+            rcoll.bs = sg_get_num(buf);
         else if (0 == strcmp(key,"bpt"))
-            rcoll.bpt = get_num(buf);
+            rcoll.bpt = sg_get_num(buf);
         else if (0 == strcmp(key,"skip"))
-            skip = get_num(buf);
+            skip = sg_get_num(buf);
         else if (0 == strcmp(key,"seek"))
-            seek = get_num(buf);
+            seek = sg_get_num(buf);
         else if (0 == strcmp(key,"count"))
-            count = get_num(buf);
+            count = sg_get_num(buf);
         else if (0 == strcmp(key,"dio"))
-            rcoll.dio = get_num(buf);
+            rcoll.dio = sg_get_num(buf);
         else if (0 == strcmp(key,"deb"))
-            rcoll.debug = get_num(buf);
+            rcoll.debug = sg_get_num(buf);
         else {
             printf("Unrecognized argument '%s'\n", key);
             usage();
