@@ -8,10 +8,11 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "sg_include.h"
-#include "sg_err.h"  /* needed for INQUIRY, if problematic then remove */
+#include "sg_lib.h"
+#include "sg_cmds.h"
 
 /* Utility program for the Linux OS SCSI generic ("sg") device driver.
-*  Copyright (C) 2000-2002 D. Gilbert
+*  Copyright (C) 2000-2005 D. Gilbert
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
@@ -19,18 +20,15 @@
 
    This shows the mapping from "sg" devices to other scsi devices
    (i.e. sd, scd or st) if any.
-   Options: -n   numeric scan: scan /dev/sg0,1,2, .... [default]
-            -a   alphabetical scan: scan /dev/sga,b,c, ....
-            -x   also show bus,chan,id,lun and type
-            -i   also show device INQUIRY information
-            -sd  only scan for sd devices [disks]
-            -st  only scan for st (and osst) devices [tapes]
-            -scd (or -sr)  only scan for scd devices [cdroms]
 
    Note: This program requires sg version 2 or better.
 
-   Version 0.18 20030812
-        - additions for osst [Kurt Garloff <garloff at suse dot de>]
+   Version 0.19 20041203
+
+   Version 1.02 20050511
+        - allow for sparse disk name with up to 3 letter SCSI
+          disk device node names (e.g. /dev/sdaaa)
+          [Nate Dailey < Nate dot Dailey at stratus dot com >]
 */
 
 
@@ -38,10 +36,14 @@
 #error "Need version 2 sg driver (linux kernel >= 2.2.6)"
 #endif
 
+static char * version_str = "1.04 20050810";
+
 static const char * devfs_id = "/dev/.devfsd";
 
 #define NUMERIC_SCAN_DEF 1   /* change to 0 to make alpha scan default */
 
+#define INQUIRY_RESP_INITIAL_LEN 36
+
 
 typedef struct my_map_info
 {
@@ -55,8 +57,9 @@ typedef struct my_map_info
 } my_map_info_t;
 
 
-#define MAX_SG_DEVS 256
-#define MAX_SD_DEVS 128
+#define MAX_SG_DEVS 2048
+#define MAX_SD_DEVS (26 + 26*26 + 26*26*26) /* sdX, sdXX, sdXXX */
+                 /* (26 + 676 + 17576) = 18278 */
 #define MAX_SR_DEVS 128
 #define MAX_ST_DEVS 128
 #define MAX_OSST_DEVS 128
@@ -84,21 +87,22 @@ static char ebuff[EBUFF_SZ];
 
 static void scan_dev_type(const char * leadin, int max_dev, int do_numeric,
                           int lin_dev_type, int last_sg_ind);
-static int do_inq(int sg_fd, int cmddt, int evpd, unsigned int pg_op,
-                  void * resp, int mx_resp_len, int noisy);
 
 static void usage()
 {
-    printf("Usage: 'sg_map [-a] [-n] [-x] [-sd] [-scd or -sr] [-st]'\n");
+    printf("Usage: 'sg_map [-a] [-h] [-i] [-n] [-sd] [-scd or -sr] [-st] "
+           "[-V] [-x]'\n");
     printf("    where: -a   do alphabetic scan (ie sga, sgb, sgc)\n");
-    printf("           -n   do numeric scan (ie sg0, sg1, sg2)\n");
-    printf("           -x   also show bus,chan,id,lun and type\n");
+    printf("           -h or -?  show this usage message then exit\n");
     printf("           -i   also show device INQUIRY strings\n");
-    printf("           -? or -h  show this usage message\n");
+    printf("           -n   do numeric scan (i.e. sg0, sg1, sg2) "
+           "(default)\n");
     printf("           -sd  show mapping to disks\n");
     printf("           -scd show mapping to cdroms (look for /dev/scd<n>\n");
     printf("           -sr  show mapping to cdroms (look for /dev/sr<n>\n");
     printf("           -st  show mapping to tapes (st and osst devices)\n");
+    printf("           -V   print version string then exit\n");
+    printf("           -x   also show bus,chan,id,lun and type\n");
     printf("    If no '-s*' arguments given then show all mappings\n");
 }
 
@@ -107,31 +111,34 @@ static void make_dev_name(char * fname, const char * leadin, int k,
                           int do_numeric)
 {
     char buff[64];
-    int  big,little;
+    int  ones,tens,hundreds; /* for lack of a better name */
+    int  buff_idx;
 
     strcpy(fname, leadin ? leadin : "/dev/sg");
     if (do_numeric) {
         sprintf(buff, "%d", k);
         strcat(fname, buff);
     }
+    else if (k >= (26 + 26*26 + 26*26*26)) {
+        strcat(fname, "xxxx");
+    }
     else {
-        if (k < 26) {
-            buff[0] = 'a' + (char)k;
-            buff[1] = '\0';
-            strcat(fname, buff);
-        }
-        else if (k <= 255) { /* assumes sequence goes x,y,z,aa,ab,ac etc */
-            big    = k/26;
-            little = k - (26 * big);
-            big    = big - 1;
+        ones = k % 26;
 
-            buff[0] = 'a' + (char)big;
-            buff[1] = 'a' + (char)little;
-            buff[2] = '\0';
-            strcat(fname, buff);
-        }
-        else
-            strcat(fname, "xxxx");
+        if ((k - 26) >= 0)
+            tens = ((k-26)/26) % 26;
+        else tens = -1;
+
+        if ((k - (26 + 26*26)) >= 0)
+             hundreds = ((k - (26 + 26*26))/(26*26)) % 26;
+        else hundreds = -1;
+
+        buff_idx = 0;
+        if (hundreds >= 0) buff[buff_idx++] = 'a' + (char)hundreds;
+        if (tens >= 0) buff[buff_idx++] = 'a' + (char)tens;
+        buff[buff_idx++] = 'a' + (char)ones;
+        buff[buff_idx] = '\0';
+        strcat(fname, buff);
     }
 }
 
@@ -167,33 +174,30 @@ int main(int argc, char * argv[])
         else if (0 == strcmp("-sd", argv[k])) {
             do_sd = 1;
             do_all_s = 0;
-        }
-        else if (0 == strcmp("-st", argv[k])) {
+        } else if (0 == strcmp("-st", argv[k])) {
             do_st = 1;
             do_osst = 1;
             do_all_s = 0;
-        }
-        else if (0 == strcmp("-sr", argv[k])) {
+        } else if (0 == strcmp("-sr", argv[k])) {
             do_sr = 1;
             do_all_s = 0;
-        }
-        else if (0 == strcmp("-scd", argv[k])) {
+        } else if (0 == strcmp("-scd", argv[k])) {
             do_scd = 1;
             do_all_s = 0;
-        }
-        else if ((0 == strcmp("-?", argv[k])) ||
-                 (0 == strncmp("-h", argv[k], 2))) {
+        } else if (0 == strcmp("-V", argv[k])) {
+            fprintf(stderr, "Version string: %s\n", version_str);
+            exit(0);
+        } else if ((0 == strcmp("-?", argv[k])) ||
+                   (0 == strncmp("-h", argv[k], 2))) {
             printf(
             "Show mapping from sg devices to other scsi device names\n\n");
             usage();
             return 1;
-        }
-        else if (*argv[k] == '-') {
+        } else if (*argv[k] == '-') {
             printf("Unknown switch: %s\n", argv[k]);
             usage();
             return 1;
-        }
-        else if (*argv[k] != '-') {
+        } else if (*argv[k] != '-') {
             printf("Unknown argument\n");
             usage();
             return 1;
@@ -203,7 +207,7 @@ int main(int argc, char * argv[])
     if (stat(devfs_id, &stat_buf) == 0)
         printf("# Note: the devfs pseudo file system is present\n");
 
-    for (k = 0, res = 0; (k < MAX_SG_DEVS)  && (num_errors < MAX_ERRORS);
+    for (k = 0, res = 0; (k < MAX_SG_DEVS) && (num_errors < MAX_ERRORS);
          ++k, res = (sg_fd >= 0) ? close(sg_fd) : 0) {
         if (res < 0) {
             snprintf(ebuff, EBUFF_SZ, "Error closing %s ", fname);
@@ -243,9 +247,10 @@ int main(int argc, char * argv[])
             continue;
         }
         if (do_inquiry) {
-            char buff[36];
+            char buff[INQUIRY_RESP_INITIAL_LEN];
 
-            if (0 == do_inq(sg_fd, 0, 0, 0, buff, sizeof(buff), 1)) {
+            if (0 == sg_ll_inquiry(sg_fd, 0, 0, 0, buff, sizeof(buff),
+                                   1, 0)) {
                 memcpy(map_arr[k].vendor, &buff[8], 8);
                 memcpy(map_arr[k].product, &buff[16], 16);
                 memcpy(map_arr[k].revision, &buff[32], 4);
@@ -273,9 +278,11 @@ int main(int argc, char * argv[])
         scan_dev_type("/dev/scd", MAX_SR_DEVS, 1, LIN_DEV_TYPE_SCD, 
                       last_sg_ind);
     if (do_all_s || do_st)
-        scan_dev_type("/dev/st", MAX_ST_DEVS, 1, LIN_DEV_TYPE_ST, last_sg_ind);
+        scan_dev_type("/dev/nst", MAX_ST_DEVS, 1, LIN_DEV_TYPE_ST,
+                      last_sg_ind);
     if (do_all_s || do_osst)
-        scan_dev_type("/dev/osst", MAX_OSST_DEVS, 1, LIN_DEV_TYPE_OSST, last_sg_ind);
+        scan_dev_type("/dev/osst", MAX_OSST_DEVS, 1, LIN_DEV_TYPE_OSST,
+                      last_sg_ind);
 
     for (k = 0; k <= last_sg_ind; ++k) {
         make_dev_name(fname, "/dev/sg", k, do_numeric);
@@ -303,7 +310,7 @@ int main(int argc, char * argv[])
                 printf("  %s", fname);
                 break;
             case LIN_DEV_TYPE_ST:
-                make_dev_name(fname, "/dev/st" , map_arr[k].oth_dev_num, 1);
+                make_dev_name(fname, "/dev/nst" , map_arr[k].oth_dev_num, 1);
                 printf("  %s", fname);
                 break;
             case LIN_DEV_TYPE_OSST:
@@ -390,20 +397,17 @@ static void scan_dev_type(const char * leadin, int max_dev, int do_numeric,
             if (EBUSY == errno) {
                 printf("Device %s is busy\n", fname);
                 ++num_errors;
-                continue;
             }
-            else if ((ENODEV == errno) || (ENOENT == errno) ||
-                     (ENXIO == errno)) {
+            else if ((ENODEV == errno) || (ENXIO == errno)) {
                 ++num_errors;
                 ++num_silent;
-                continue;
             }
-            else {
+            else if (ENOENT != errno) { /* ignore ENOENT for sparse names */
                 snprintf(ebuff, EBUFF_SZ, "Error opening %s ", fname);
                 perror(ebuff);
                 ++num_errors;
-                continue;
             }
+            continue;
         }
 
         res = ioctl(sg_fd, SCSI_IOCTL_GET_IDLUN, &my_idlun);
@@ -444,53 +448,3 @@ static void scan_dev_type(const char * leadin, int max_dev, int do_numeric,
     }
 }
 
-#define SENSE_BUFF_LEN 32       /* Arbitrary, could be larger */
-#define DEF_TIMEOUT 60000       /* 60,000 millisecs == 60 seconds */
-
-#define INQUIRY_CMD     0x12
-#define INQUIRY_CMDLEN  6
-
-static int do_inq(int sg_fd, int cmddt, int evpd, unsigned int pg_op, 
-                  void * resp, int mx_resp_len, int noisy)
-{
-    int res;
-    unsigned char inqCmdBlk[INQUIRY_CMDLEN] = {INQUIRY_CMD, 0, 0, 0, 0, 0};
-    unsigned char sense_b[SENSE_BUFF_LEN];
-    struct sg_io_hdr io_hdr;
-
-    if (cmddt)
-        inqCmdBlk[1] |= 2;
-    if (evpd)
-        inqCmdBlk[1] |= 1;
-    inqCmdBlk[2] = (unsigned char)pg_op;
-    inqCmdBlk[4] = (unsigned char)mx_resp_len;
-    memset(&io_hdr, 0, sizeof(struct sg_io_hdr));
-    io_hdr.interface_id = 'S';
-    io_hdr.cmd_len = sizeof(inqCmdBlk);
-    io_hdr.mx_sb_len = sizeof(sense_b);
-    io_hdr.dxfer_direction = SG_DXFER_FROM_DEV;
-    io_hdr.dxfer_len = mx_resp_len;
-    io_hdr.dxferp = resp;
-    io_hdr.cmdp = inqCmdBlk;
-    io_hdr.sbp = sense_b;
-    io_hdr.timeout = DEF_TIMEOUT;
-
-    if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
-        perror("SG_IO (inquiry) error");
-        return -1;
-    }
-    res = sg_err_category3(&io_hdr);
-    switch (res) {
-    case SG_ERR_CAT_CLEAN:
-    case SG_ERR_CAT_RECOVERED:
-        return 0;
-    default:
-        if (noisy) {
-            char ebuff[EBUFF_SZ];
-            snprintf(ebuff, EBUFF_SZ, "Inquiry error, CmdDt=%d, "
-                     "EVPD=%d, page_opcode=%x ", cmddt, evpd, pg_op);
-            sg_chk_n_print3(ebuff, &io_hdr);
-        }
-        return -1;
-    }
-}
