@@ -10,10 +10,11 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "sg_include.h"
-#include "sg_err.h"
+#include "sg_lib.h"
+#include "sg_cmds.h"
 
 /* A utility program for the Linux OS SCSI subsystem.
-*  Copyright (C) 2004 D. Gilbert
+*  Copyright (C) 2004-2005 D. Gilbert
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
@@ -23,7 +24,7 @@
 
 */
 
-static char * version_str = "0.17 20040809";
+static char * version_str = "0.23 20050808";
 
 
 #define SENSE_BUFF_LEN 32       /* Arbitrary, could be larger */
@@ -46,9 +47,6 @@ static char * version_str = "0.17 20040809";
 #define PROUT_REG_MOVE_SA 0x7
 #define MX_ALLOC_LEN 8192
 
-#define INQUIRY_CMD 0x12
-#define INQUIRY_CMDLEN 6
-
 #define EBUFF_SZ 256
 
 
@@ -154,16 +152,18 @@ static int do_prin(int sg_fd, int rq_servact, void * resp, int mx_resp_len,
     }
     res = sg_err_category3(&io_hdr);
     switch (res) {
-    case SG_ERR_CAT_CLEAN:
-    case SG_ERR_CAT_RECOVERED:
+    case SG_LIB_CAT_RECOVERED:
+        sg_chk_n_print3("PRIN, continuing", &io_hdr, verbose);
+        /* fall through */
+    case SG_LIB_CAT_CLEAN:
         return 0;
     default:
-        if (noisy) {
+        if (noisy || verbose) {
             char ebuff[EBUFF_SZ];
             snprintf(ebuff, EBUFF_SZ, "PRIN error, service_action: %s",
                      ((rq_servact < num_prin_sa_strs) ? 
                         prin_sa_strs[rq_servact] : "??"));
-            sg_chk_n_print3(ebuff, &io_hdr);
+            sg_chk_n_print3(ebuff, &io_hdr, verbose);
         }
         return -1;
     }
@@ -215,8 +215,10 @@ static int do_prout(int sg_fd, int rq_servact, int rq_scope,
     }
     res = sg_err_category3(&io_hdr);
     switch (res) {
-    case SG_ERR_CAT_CLEAN:
-    case SG_ERR_CAT_RECOVERED:
+    case SG_LIB_CAT_RECOVERED:
+        sg_chk_n_print3("PROUT, continuing", &io_hdr, verbose);
+        /* fall through */
+    case SG_LIB_CAT_CLEAN:
         return 0;
     default:
         if (noisy) {
@@ -224,58 +226,7 @@ static int do_prout(int sg_fd, int rq_servact, int rq_scope,
             snprintf(ebuff, EBUFF_SZ, "PROUT error, service_action: %s",
                      ((rq_servact < num_prout_sa_strs) ? 
                         prout_sa_strs[rq_servact] : "??"));
-            sg_chk_n_print3(ebuff, &io_hdr);
-        }
-        return -1;
-    }
-}
-
-/* Returns 0 when successful, else -1 */
-static int do_simple_inq(int sg_fd, int noisy, 
-                         char * resp_data, int verbose)
-{
-    int res, k;
-    unsigned char inqCmdBlk[INQUIRY_CMDLEN] = {INQUIRY_CMD, 0, 0, 0, 0, 0};
-    unsigned char sense_b[SENSE_BUFF_LEN];
-    struct sg_io_hdr io_hdr;
-    const int resp_sz = 36;
-
-    if (! resp_data)
-        return -1;
-    memset(resp_data, 0, resp_sz);
-    resp_data[0] = 0x7f;    /* peri_qual=3, peri_type=1f */
-    inqCmdBlk[4] = resp_sz;
-    if (verbose) {
-        fprintf(stderr, "        inquiry cdb: ");
-        for (k = 0; k < INQUIRY_CMDLEN; ++k)
-            fprintf(stderr, "%02x ", inqCmdBlk[k]);
-        fprintf(stderr, "\n");
-    }
-    memset(&io_hdr, 0, sizeof(struct sg_io_hdr));
-    io_hdr.interface_id = 'S';
-    io_hdr.cmd_len = sizeof(inqCmdBlk);
-    io_hdr.mx_sb_len = sizeof(sense_b);
-    io_hdr.dxfer_direction = SG_DXFER_FROM_DEV;
-    io_hdr.dxfer_len = resp_sz;
-    io_hdr.dxferp = resp_data;
-    io_hdr.cmdp = inqCmdBlk;
-    io_hdr.sbp = sense_b;
-    io_hdr.timeout = DEF_TIMEOUT;
-
-    if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
-        perror("SG_IO (inquiry) error");
-        return -1;
-    }
-    res = sg_err_category3(&io_hdr);
-    switch (res) {
-    case SG_ERR_CAT_CLEAN:
-    case SG_ERR_CAT_RECOVERED:
-        return 0;
-    default:
-        if (noisy) {
-            char ebuff[EBUFF_SZ];
-            snprintf(ebuff, EBUFF_SZ, "Inquiry error ");
-            sg_chk_n_print3(ebuff, &io_hdr);
+            sg_chk_n_print3(ebuff, &io_hdr, verbose);
         }
         return -1;
     }
@@ -313,7 +264,8 @@ static void usage()
             "       --register|-G          PR Out: Register\n"
             "       --register-ignore|-I   PR Out: Register and Ignore\n"
             "       --register-move|-M     PR Out: Register and Move\n"
-            "       --relative-target-port=<h>|-Q  PR Out parameter for '-M'\n"
+            "       --relative-target-port=<h>|-Q <h>  PR Out parameter for "
+            "'-M'\n"
             "       --release|-L           PR Out: Release\n"
             "       --report-capabilities|-c   PR In: Report Capabilities\n"
             "       --reserve|-R           PR Out: Reserve\n"
@@ -324,7 +276,8 @@ static void usage()
             "       --unreg|-U     optional with PR Out Register and Move\n"
             "       --verbose|-v   output additional debug information\n"
             "       --version|-V   output version string\n"
-            "       -?   output this usage message\n");
+            "       -?   output this usage message\n\n"
+            "Performs a PERSISTENT RESERVATION (IN or OUT) SCSI command\n");
 }
 
 static const char * scsi_ptype_strs[] = {
@@ -347,8 +300,19 @@ static const char * scsi_ptype_strs[] = {
     /* 0x10 */ "bridging expander",
     "object based storage",
     "automation/driver interface",
+    "0x13", "0x14", "0x15", "0x16", "0x17", "0x18",
+    "0x19", "0x1a", "0x1b", "0x1c", "0x1d",
+    "well known logical unit",
+    "no physical device on this lu",
 };
 
+static const char * get_ptype_str(int scsi_ptype)
+{
+    int num = sizeof(scsi_ptype_strs) / sizeof(scsi_ptype_strs[0]);
+
+    return (scsi_ptype < num) ? scsi_ptype_strs[scsi_ptype] : "";
+}
+
 static const char * pr_type_strs[] = {
     "obsolete [0]",
     "Write Exclusive",
@@ -363,7 +327,8 @@ static const char * pr_type_strs[] = {
     "obsolete [0xd]", "obsolete [0xe]", "obsolete [0xf]",
 };
 
-static void decode_transport_id(unsigned char * ucp, int len)
+static void decode_transport_id(const char * leadin, unsigned char * ucp,
+                                int len)
 {
     int format_code, proto_id, num, j, k;
     unsigned long long ull;
@@ -371,58 +336,62 @@ static void decode_transport_id(unsigned char * ucp, int len)
 
     for (k = 0, bump; k < len; k += bump, ucp += bump) {
         if ((len < 24) || (0 != (len % 4)))
-            printf("      Transport Id short or not multiple of 4 "
-                   "[length=%d]:\n", len);
+            printf("%sTransport Id short or not multiple of 4 "
+                   "[length=%d]:\n", leadin, len);
         else
-            printf("      Transport Id of initiator:\n");
+            printf("%sTransport Id of initiator:\n", leadin);
         format_code = ((ucp[0] >> 6) & 0x3);
         proto_id = (ucp[0] & 0xf);
         switch (proto_id) {
         case 0: /* Fibre channel */
-            printf("        FCP-2 World Wide Name:\n");
+            printf("%s  FCP-2 World Wide Name:\n", leadin);
             if (0 != format_code) 
-                printf("        [Unexpected format code: %d]\n", format_code);
+                printf("%s  [Unexpected format code: %d]\n", leadin,
+                       format_code);
             dStrHex((const char *)&ucp[8], 8, 0);
             bump = 24;
             break;
         case 1: /* Parallel SCSI */
-            printf("        Parallel SCSI initiator SCSI address: 0x%x\n",
-                   ((ucp[2] << 8) | ucp[3]));
+            printf("%s  Parallel SCSI initiator SCSI address: 0x%x\n",
+                   leadin, ((ucp[2] << 8) | ucp[3]));
             if (0 != format_code) 
-                printf("        [Unexpected format code: %d]\n", format_code);
-            printf("        relative port number (of target): 0x%x\n",
-                   ((ucp[6] << 8) | ucp[7]));
+                printf("%s  [Unexpected format code: %d]\n", leadin,
+                       format_code);
+            printf("%s  relative port number (of corresponding target): "
+                   "0x%x\n", leadin, ((ucp[6] << 8) | ucp[7]));
             bump = 24;
             break;
         case 2: /* SSA */
-            printf("        SSA (transport id not defined):\n");
-            printf("        format code: %d\n", format_code);
+            printf("%s  SSA (transport id not defined):\n", leadin);
+            printf("%s  format code: %d\n", leadin, format_code);
             dStrHex((const char *)ucp, ((len > 24) ? 24 : len), 0);
             bump = 24;
             break;
         case 3: /* IEEE 1394 */
-            printf("        IEEE 1394 EUI-64 name:\n");
+            printf("%s  IEEE 1394 EUI-64 name:\n", leadin);
             if (0 != format_code) 
-                printf("        [Unexpected format code: %d]\n", format_code);
+                printf("%s  [Unexpected format code: %d]\n", leadin,
+                       format_code);
             dStrHex((const char *)&ucp[8], 8, 0);
             bump = 24;
             break;
         case 4: /* Remote Direct Memory Access (RDMA) */
-            printf("        RDMA initiator port identifier:\n");
+            printf("%s  RDMA initiator port identifier:\n", leadin);
             if (0 != format_code) 
-                printf("        [Unexpected format code: %d]\n", format_code);
+                printf("%s  [Unexpected format code: %d]\n", leadin,
+                       format_code);
             dStrHex((const char *)&ucp[8], 16, 0);
             bump = 24;
             break;
         case 5: /* iSCSI */
-            printf("        iSCSI ");
+            printf("%s  iSCSI ", leadin);
             num = ((ucp[2] << 8) | ucp[3]);
             if (0 == format_code)
                 printf("name: %.*s\n", num, &ucp[4]);
             else if (1 == format_code)
                 printf("world wide unique port id: %.*s\n", num, &ucp[4]);
             else {
-                printf("        [Unexpected format code: %d]\n", format_code);
+                printf("  [Unexpected format code: %d]\n", format_code);
                 dStrHex((const char *)ucp, num + 4, 0);
             }
             bump = (((num + 4) < 24) ? 24 : num + 4);
@@ -434,26 +403,27 @@ static void decode_transport_id(unsigned char * ucp, int len)
                     ull <<= 8;
                 ull |= ucp[4 + j];
             }
-            printf("        SAS address: 0x%llx\n", ull);
+            printf("%s  SAS address: 0x%llx\n", leadin, ull);
             if (0 != format_code) 
-                printf("        [Unexpected format code: %d]\n", format_code);
+                printf("%s  [Unexpected format code: %d]\n", leadin,
+                       format_code);
             bump = 24;
             break;
-        case 7: /* Automation/Drive Interface Transport Protocol */
-            printf("        ADT:\n");
-            printf("        format code: %d\n", format_code);
+        case 7: /* Automation/Drive Interface */
+            printf("%s  ADT:\n", leadin);
+            printf("%s  format code: %d\n", leadin, format_code);
             dStrHex((const char *)ucp, ((len > 24) ? 24 : len), 0);
             bump = 24;
             break;
         case 8: /* ATAPI */
-            printf("        ATAPI:\n");
-            printf("        format code: %d\n", format_code);
+            printf("%s  ATAPI:\n", leadin);
+            printf("%s  format code: %d\n", leadin, format_code);
             dStrHex((const char *)ucp, ((len > 24) ? 24 : len), 0);
             bump = 24;
             break;
         default:
-            fprintf(stderr, "        unknown protocol id=0x%x  "
-                    "format_code=%d\n", proto_id, format_code);
+            fprintf(stderr, "%s  unknown protocol id=0x%x  "
+                    "format_code=%d\n", leadin, proto_id, format_code);
             dStrHex((const char *)ucp, ((len > 24) ? 24 : len), 0);
             bump = 24;
             break;
@@ -605,7 +575,7 @@ static int prin_work(int sg_fd, int prin_sa, int do_verbose, int do_hex)
                 } else
                     printf("      not reservation holder\n");
                 if (add_desc_len > 0)
-                    decode_transport_id(&ucp[24], add_desc_len);
+                    decode_transport_id("      ", &ucp[24], add_desc_len);
             }
         }
     }
@@ -704,7 +674,8 @@ static int prout_rmove_work(int sg_fd, unsigned int prout_type,
 }
 
 static int build_transportid(const char * inp, unsigned char * tid_arr,
-                             int * tid_arr_len, int * num_tids)
+                             int * tid_arr_len, int * num_tids,
+                             int max_arr_len)
 {
     int in_len, k, j, m;
     unsigned int h;
@@ -747,7 +718,7 @@ static int build_transportid(const char * inp, unsigned char * tid_arr,
             if ('#' == *lcp)
                 continue;
             k = strspn(lcp, "0123456789aAbBcCdDeEfF ,\t");
-            if (in_len != k) {
+            if ((k < in_len) && ('#' != lcp[k])) {
                 fprintf(stderr, "build_transportid: syntax error at "
                         "line %d, pos %d\n", j + 1, m + k + 1);
                 return 1;
@@ -757,7 +728,12 @@ static int build_transportid(const char * inp, unsigned char * tid_arr,
                     if (h > 0xff) {
                         fprintf(stderr, "build_transportid: hex number "
                                 "larger than 0xff in line %d, pos %d\n",
-                                j + 1, lcp - line + 1);
+                                j + 1, (int)(lcp - line + 1));
+                        return 1;
+                    }
+                    if ((off + k) >= max_arr_len) {
+                        fprintf(stderr, "build_transportid: array length "
+                                "exceeded\n");
                         return 1;
                     }
                     tid_arr[off + k] = h;
@@ -765,10 +741,16 @@ static int build_transportid(const char * inp, unsigned char * tid_arr,
                     if (NULL == lcp)
                         break;
                     lcp += strspn(lcp, " ,\t");
+                    if ('\0' == *lcp)
+                        break;
                 } else {
+                    if ('#' == *lcp) {
+                        --k;
+                        break;
+                    }
                     fprintf(stderr, "build_transportid: error in "
                             "line %d, at pos %d\n", j + 1,
-                            lcp - line + 1);
+                            (int)(lcp - line + 1));
                     return 1;
                 }
             }
@@ -789,11 +771,11 @@ static int build_transportid(const char * inp, unsigned char * tid_arr,
                     k + 1);
             return 1;
         }
-        for (k = 0; k < 1024; ++k) {
+        for (k = 0; k < max_arr_len; ++k) {
             if (1 == sscanf(lcp, "%x", &h)) {
                 if (h > 0xff) {
                     fprintf(stderr, "build_transportid: hex number larger "
-                            "than 0xff at pos %d\n", lcp - inp + 1);
+                            "than 0xff at pos %d\n", (int)(lcp - inp + 1));
                     return 1;
                 }
                 tid_arr[k] = h;
@@ -803,7 +785,7 @@ static int build_transportid(const char * inp, unsigned char * tid_arr,
                 lcp = cp + 1;
             } else {
                 fprintf(stderr, "build_transportid: error at pos %d\n",
-                        lcp - inp + 1);
+                        (int)(lcp - inp + 1));
                 return 1;
             }
         }
@@ -814,6 +796,10 @@ static int build_transportid(const char * inp, unsigned char * tid_arr,
         *tid_arr_len = k;
         if (num_tids)
             *num_tids = 1;
+        if (k >= max_arr_len) {
+            fprintf(stderr, "build_transportid: array length exceeded\n");
+            return 1;
+        }
     }
     return 0;
 }
@@ -828,7 +814,6 @@ int main(int argc, char * argv[])
     unsigned int param_rtp = 0;
     char device_name[256];
     char ebuff[EBUFF_SZ];
-    char inq_buff[36];
     int num_prin_sa = 0;
     int num_prout_sa = 0;
     int num_prout_param = 0;
@@ -848,6 +833,8 @@ int main(int argc, char * argv[])
     unsigned char transportid_arr[MX_ALLOC_LEN];
     int transportid_arr_len = 0;
     int num_transportids = 0;
+    struct sg_simple_inquiry_resp inq_resp;
+    const char * cp;
 
     device_name[0] = '\0';
     while (1) {
@@ -972,7 +959,8 @@ int main(int argc, char * argv[])
             memset(transportid_arr, 0, sizeof(transportid_arr));
             if (0 != build_transportid(optarg, transportid_arr, 
                                        &transportid_arr_len, 
-                                       &num_transportids)) {
+                                       &num_transportids,
+                                       sizeof(transportid_arr))) {
                 fprintf(stderr, "bad argument to '--transport-id'\n");
                 return 1;
             }
@@ -1072,7 +1060,7 @@ int main(int argc, char * argv[])
         fprintf(stderr, "number of tranport-ids decoded from "
                 "command line (or stdin): %d\n", num_transportids);
         fprintf(stderr, "  Decode given transport-ids:\n");
-        decode_transport_id(transportid_arr, transportid_arr_len);
+        decode_transport_id("      ", transportid_arr, transportid_arr_len);
     }
 
     if (do_inquiry) {
@@ -1082,16 +1070,15 @@ int main(int argc, char * argv[])
             perror(ebuff);
             return 1;
         }
-        if (0 == do_simple_inq(sg_fd, 1, inq_buff, do_verbose)) {
-            printf("  %.8s  %.16s  %.4s\n", inq_buff + 8, inq_buff + 16,
-                   inq_buff + 32);
-            peri_type = inq_buff[0] & 0x1f;
-            if (peri_type >= 
-                   (int)(sizeof(scsi_ptype_strs) / sizeof(scsi_ptype_strs[0])))
-                printf("  Peripheral device type: 0x%x\n", peri_type);
+        if (0 == sg_simple_inquiry(sg_fd, &inq_resp, 1, do_verbose)) {
+            printf("  %.8s  %.16s  %.4s\n", inq_resp.vendor, inq_resp.product,
+                   inq_resp.revision);
+            peri_type = inq_resp.peripheral_type;
+            cp = get_ptype_str(peri_type);
+            if (strlen(cp) > 0)
+                printf("  Peripheral device type: %s\n", cp);
             else
-                printf("  Peripheral device type: %s\n", 
-                       scsi_ptype_strs[peri_type]);
+                printf("  Peripheral device type: 0x%x\n", peri_type);
         } else {
             printf("sg_persist: %s doesn't respond to a SCSI INQUIRY\n", 
                    device_name);
