@@ -13,40 +13,27 @@
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <sys/time.h>
-#include <linux/fs.h>
 #include "sg_include.h"
-#include "sg_err.h"
+#include "sg_lib.h"
 
 /* Test code for D. Gilbert's extensions to the Linux OS SCSI generic ("sg")
    device driver.
-*  Copyright (C) 1999-2004 D. Gilbert
+*  Copyright (C) 1999-2005 D. Gilbert
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 
-   This program uses the SCSI command READ BUFFER on the given sg
+   This program uses the SCSI command READ BUFFER on the given
    device, first to find out how big it is and then to read that
-   buffer. The '-q' option skips the data transfer from the kernel
-   DMA buffers to the user space. The '-b=num' option allows the
-   buffer size (in KBytes) to be specified (default is to use the
-   number obtained from READ BUFFER (descriptor) SCSI command).
-   The '-s=num' option allows the total size of the transfer to be
-   set (in megabytes, the default is 200 MB). The '-d' option requests
-   direct io (and is overridden by '-q').
-   The '-m' option request mmap-ed IO (and overrides the '-q' and '-d'
-   options if they are also given).
-   The ability to time transfers internally (based on gettimeofday()) has
-   been added with the '-t' option.
-
-   Version 4.75 (20040809)
+   buffer (data mode, buffer id 0).
 */
 
 
 #define RB_MODE_DESC 3
 #define RB_MODE_DATA 2
 #define RB_DESC_LEN 4
-#define RB_MB_TO_READ 200
+#define RB_MIB_TO_READ 200
 #define RB_OPCODE 0x3C
 #define RB_CMD_LEN 10
 
@@ -58,10 +45,29 @@
 
 #define ME "sg_rbuf: "
 
+static char * version_str = "4.83 20050808";
+
+static void usage()
+{
+    printf("Usage: sg_rbuf [-b=num] [[-q] | [-d] | [-m]] [-s=num] [-t] "
+           "[-v] [-V]\n               <scsi_device>\n");
+    printf("  where  -b=num   num is buffer size to use (in KiB)\n");
+    printf("         -d       requests dio ('-q' overrides it)\n");
+    printf("         -m       requests mmap-ed IO (overrides -q, -d)\n");
+    printf("         -q       quick, don't xfer to user space\n");
+    printf("         -s=num   num is total size to read (in MiB)\n");
+    printf("                    default total size is 200 MiB\n");
+    printf("                    max total size is 4000 MiB\n");
+    printf("         -t       time the data transfer\n");
+    printf("         -v       increase verbosity (more debug)\n");
+    printf("         -V       print version string then exit\n\n");
+    printf("Use SCSI READ BUFFER command (data mode, buffer id 0) "
+           "repeatedly\n");
+}
 
 int main(int argc, char * argv[])
 {
-    int sg_fd, res, j, m;
+    int sg_fd, res, j, m, plen, jmp_out;
     unsigned int k, num;
     unsigned char rbCmdBlk [RB_CMD_LEN];
     unsigned char * rbBuff = NULL;
@@ -72,9 +78,11 @@ int main(int argc, char * argv[])
     int do_dio = 0;
     int do_mmap = 0;
     int do_time = 0;
+    int verbose = 0;
     int buf_size = 0;
-    unsigned int total_size_mb = RB_MB_TO_READ;
-    char * file_name = 0;
+    unsigned int total_size_mib = RB_MIB_TO_READ;
+    const char * file_name = 0;
+    const char * cp;
     size_t psz = getpagesize();
     int dio_incomplete = 0;
     struct sg_io_hdr io_hdr;
@@ -84,56 +92,82 @@ int main(int argc, char * argv[])
 #endif
 
     for (j = 1; j < argc; ++j) {
-        if (0 == strncmp("-b=", argv[j], 3)) {
-            m = 3;
-            num = sscanf(argv[j] + m, "%d", &buf_size);
-            if ((1 != num) || (buf_size <= 0)) {
-                printf("Couldn't decode number after '-b' switch\n");
-                file_name = 0;
-                break;
+        cp = argv[j];
+        plen = strlen(cp);
+        if (plen <= 0)
+            continue;
+        if ('-' == *cp) {
+            for (--plen, ++cp, jmp_out = 0; plen > 0; --plen, ++cp) {
+                switch (*cp) {
+                case 'd':
+                    do_dio = 1;
+                    break;
+                case 'm':
+                    do_mmap = 1;
+                    break;
+                case 'q':
+                    do_quick = 1;
+                    break;
+                case 't':
+                    do_time = 1;
+                    break;
+                case 'v':
+                    ++verbose;
+                    break;
+                case 'V':
+                    fprintf(stderr, "Version string: %s\n", version_str);
+                    exit(0);
+                case '?':
+                    usage();
+                    return 1;
+                default:
+                    jmp_out = 1;
+                    break;
+                }
+                if (jmp_out)
+                    break;
             }
-            buf_size *= 1024;
-        }
-        else if (0 == strncmp("-s=", argv[j], 3)) {
-            m = 3;
-            num = sscanf(argv[j] + m, "%u", &total_size_mb);
-            if (1 != num) {
-                printf("Couldn't decode number after '-s' switch\n");
-                file_name = 0;
-                break;
+            if (plen <= 0)
+                continue;
+            if (0 == strncmp("b=", cp, 2)) {
+                m = 2;
+                num = sscanf(cp + m, "%d", &buf_size);
+                if ((1 != num) || (buf_size <= 0)) {
+                    printf("Couldn't decode number after 'b=' option\n");
+                    usage();
+                    return 1;
+                }
+                buf_size *= 1024;
             }
+            else if (0 == strncmp("s=", cp, 2)) {
+                m = 2;
+                num = sscanf(cp + m, "%u", &total_size_mib);
+                if (1 != num) {
+                    printf("Couldn't decode number after 's=' option\n");
+                    usage();
+                    return 1;
+                }
+            } else if (jmp_out) {
+                fprintf(stderr, "Unrecognized option: %s\n", cp);
+                usage();
+                return 1;
+            }
+        } else if (0 == file_name)
+            file_name = cp;
+        else {
+            fprintf(stderr, "too many arguments, got: %s, not expecting: "
+                    "%s\n", file_name, cp);
+            usage();
+            return 1;
         }
-        else if (0 == strcmp("-q", argv[j]))
-            do_quick = 1;
-        else if (0 == strcmp("-d", argv[j]))
-            do_dio = 1;
-        else if (0 == strcmp("-m", argv[j]))
-            do_mmap = 1;
-        else if (0 == strcmp("-t", argv[j]))
-            do_time = 1;
-        else if (*argv[j] == '-') {
-            printf("Unrecognized switch: %s\n", argv[j]);
-            file_name = 0;
-            break;
-        }
-        else
-            file_name = argv[j];
     }
     if (0 == file_name) {
-        printf("Usage: sg_rbuf [[-q] | [-d] | [-m]] [-t] [-b=num] [-s=num] "
-               "<generic_device>\n");
-        printf("  where: -q       quick, don't xfer to user space\n");
-        printf("         -d       requests dio ('-q' overrides it)\n");
-        printf("         -m       requests mmap-ed IO (overrides -q, -d)\n");
-        printf("         -t       time the data transfer\n");
-        printf("         -b=num   num is buffer size to use (in KBytes)\n");
-        printf("         -s=num   num is total size to read (in MBytes)\n");
-        printf("                    default total size is 200 MBytes\n");
-        printf("                    max total size is 4000 MBytes\n");
+        fprintf(stderr, "No <scsi_device> argument given\n");
+        usage();
         return 1;
     }
 
-    sg_fd = open(file_name, O_RDONLY);
+    sg_fd = open(file_name, O_RDONLY | O_NONBLOCK);
     if (sg_fd < 0) {
         perror(ME "open error");
         return 1;
@@ -151,7 +185,7 @@ int main(int argc, char * argv[])
 
     memset(rbCmdBlk, 0, RB_CMD_LEN);
     rbCmdBlk[0] = RB_OPCODE;
-    rbCmdBlk[1] = RB_MODE_DESC;
+    rbCmdBlk[1] = RB_MODE_DESC; /* data mode, buffer id 0 */
     rbCmdBlk[8] = RB_DESC_LEN;
     memset(&io_hdr, 0, sizeof(struct sg_io_hdr));
     io_hdr.interface_id = 'S';
@@ -163,23 +197,32 @@ int main(int argc, char * argv[])
     io_hdr.cmdp = rbCmdBlk;
     io_hdr.sbp = sense_buffer;
     io_hdr.timeout = 60000;     /* 60000 millisecs == 60 seconds */
-    /* do normal IO to find RB size (not dio or mmap-ed at this stage) */
+    if (verbose) {
+        fprintf(stderr, "    Read buffer cdb: ");
+        for (k = 0; k < RB_CMD_LEN; ++k)
+            fprintf(stderr, "%02x ", rbCmdBlk[k]);
+        fprintf(stderr, "\n");
+    }
 
+    /* do normal IO to find RB size (not dio or mmap-ed at this stage) */
     if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
         perror(ME "SG_IO READ BUFFER descriptor error");
         if (rawp) free(rawp);
         return 1;
     }
 
+    if (verbose > 2)
+        fprintf(stderr, "      duration=%u ms\n", io_hdr.duration);
     /* now for the error processing */
     switch (sg_err_category3(&io_hdr)) {
-    case SG_ERR_CAT_CLEAN:
-        break;
-    case SG_ERR_CAT_RECOVERED:
-        printf("Recovered error on READ BUFFER descriptor, continuing\n");
+    case SG_LIB_CAT_RECOVERED:
+        sg_chk_n_print3("READ BUFFER descriptor, continuing", &io_hdr,
+                        verbose);
+        /* fall through */
+    case SG_LIB_CAT_CLEAN:
         break;
     default: /* won't bother decoding other categories */
-        sg_chk_n_print3("READ BUFFER descriptor error", &io_hdr);
+        sg_chk_n_print3("READ BUFFER descriptor error", &io_hdr, verbose);
         if (rawp) free(rawp);
         return 1;
     }
@@ -215,7 +258,7 @@ int main(int argc, char * argv[])
         if (MAP_FAILED == rbBuff) {
             if (ENOMEM == errno)
                 printf(ME "mmap() out of memory, try a smaller "
-                       "buffer size than %d KB\n", buf_size / 1024);
+                       "buffer size than %d KiB\n", buf_size / 1024);
             else
                 perror(ME "error using mmap()");
             return 1;
@@ -234,7 +277,7 @@ int main(int argc, char * argv[])
             rbBuff = rawp;
     }
 
-    num = (total_size_mb * 1024U * 1024U) / (unsigned int)buf_size;
+    num = (total_size_mib * 1024U * 1024U) / (unsigned int)buf_size;
     if (do_time) {
         start_tm.tv_sec = 0;
         start_tm.tv_usec = 0;
@@ -270,26 +313,35 @@ int main(int argc, char * argv[])
             io_hdr.flags |= SG_FLAG_DIRECT_IO;
         else if (do_quick)
             io_hdr.flags |= SG_FLAG_NO_DXFER;
+        if (verbose > 1) {
+            fprintf(stderr, "    Read buffer cdb: ");
+            for (k = 0; k < RB_CMD_LEN; ++k)
+                fprintf(stderr, "%02x ", rbCmdBlk[k]);
+            fprintf(stderr, "\n");
+        }
 
         if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
             if (ENOMEM == errno)
                 printf(ME "SG_IO data; out of memory, try a smaller "
-                       "buffer size than %d KB\n", buf_size / 1024);
+                       "buffer size than %d KiB\n", buf_size / 1024);
             else
                 perror(ME "SG_IO READ BUFFER data error");
             if (rawp) free(rawp);
             return 1;
         }
 
+        if (verbose > 2)
+            fprintf(stderr, "      duration=%u ms\n",
+                    io_hdr.duration);
         /* now for the error processing */
         switch (sg_err_category3(&io_hdr)) {
-        case SG_ERR_CAT_CLEAN:
+        case SG_LIB_CAT_CLEAN:
             break;
-        case SG_ERR_CAT_RECOVERED:
-            printf("Recovered error on READ BUFFER data, continuing\n");
+        case SG_LIB_CAT_RECOVERED:
+            sg_chk_n_print3("READ BUFFER data, continuing", &io_hdr, verbose);
             break;
         default: /* won't bother decoding other categories */
-            sg_chk_n_print3("READ BUFFER data error", &io_hdr);
+            sg_chk_n_print3("READ BUFFER data error", &io_hdr, verbose);
             if (rawp) free(rawp);
             return 1;
         }
@@ -331,8 +383,8 @@ int main(int argc, char * argv[])
     }
     if (dio_incomplete)
         printf(">> direct IO requested but not done\n");
-    printf("Read %u MBytes (actual %u MB, %u bytes), buffer size=%d KBytes\n",
-           total_size_mb, (num * buf_size) / 1048576, num * buf_size,
+    printf("Read %u MiB (actual %u MiB, %u bytes), buffer size=%d KiB\n",
+           total_size_mib, (num * buf_size) / 1048576, num * buf_size,
            buf_size / 1024);
 
     if (rawp) free(rawp);
