@@ -10,11 +10,11 @@
 #include <sys/stat.h>
 #include <scsi/scsi_ioctl.h>
 #include "sg_include.h"
-#include "sg_err.h"
+#include "sg_lib.h"
 
 /* Test code for D. Gilbert's extensions to the Linux OS SCSI generic ("sg")
    device driver.
-*  Copyright (C) 1999 - 2004 D. Gilbert
+*  Copyright (C) 1999 - 2005 D. Gilbert
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
@@ -38,7 +38,7 @@
    F. Jansen - modification to extend beyond 26 sg devices.
 */
 
-static char * version_str = "4.00 20040622";
+static char * version_str = "4.05 20050905";
 
 #define ME "sg_scan: "
 
@@ -79,15 +79,17 @@ static unsigned char inqCmdBlk[INQ_CMD_LEN] =
 
 void usage()
 {
-    printf("Usage: 'sg_scan [-a] [-i] [-n] [-V] [-w] [-x] [<sam_dev>]*'\n");
+    printf("Usage: 'sg_scan [-a] [-i] [-n] [-v] [-V] [-w] [-x] "
+           "[<sam_dev>]*'\n");
     printf("    where: -a   do alpha scan (ie sga, sgb, sgc)\n");
     printf("           -i   do SCSI INQUIRY, output results\n");
     printf("           -n   do numeric scan (ie sg0, sg1...) [default]\n");
+    printf("           -v   increase verbosity\n");
     printf("           -V   output version string then exit\n");
     printf("           -w   force open with read/write flag\n");
     printf("           -x   extra information output about queuing\n");
     printf("      <sam_dev> name of device that understands SAM command"
-	   " set\n");
+           " set\n");
 }
 
 void make_dev_name(char * fname, int k, int do_numeric)
@@ -124,11 +126,12 @@ void make_dev_name(char * fname, int k, int do_numeric)
 
 int main(int argc, char * argv[])
 {
-    int sg_fd, res, k, j, f;
+    int sg_fd, res, k, j, f, plen, jmp_out;
     unsigned char inqBuff[INQ_REPLY_LEN];
     int do_numeric = NUMERIC_SCAN_DEF;
     int do_inquiry = 0;
     int do_extra = 0;
+    int verbose = 0;
     int writeable = 0;
     int num_errors = 0;
     int num_silent = 0;
@@ -144,69 +147,94 @@ int main(int argc, char * argv[])
     int has_file_args = 0;
     const int max_file_args = 2048;
     int * argv_index_arr;
+    const char * cp;
 
     if ((argv_index_arr = malloc(max_file_args * sizeof(int))))
-	memset(argv_index_arr, 0, max_file_args * sizeof(int));
+        memset(argv_index_arr, 0, max_file_args * sizeof(int));
     else {
         printf(ME "Out of memory\n");
         return 1;
     }
 
     for (k = 1, j = 0; k < argc; ++k) {
-        if (0 == strcmp("-a", argv[k]))
-            do_numeric = 0;
-        else if (0 == strcmp("-i", argv[k]))
-            do_inquiry = 1;
-        else if (0 == strcmp("-n", argv[k]))
-            do_numeric = 1;
-        else if (0 == strcmp("-V", argv[k])) {
-            printf("Version string: %s\n", version_str);
-            exit(0);
-        } else if (0 == strcmp("-w", argv[k]))
-            writeable = 1;
-        else if (0 == strcmp("-x", argv[k]))
-            do_extra = 1;
-        else if ((0 == strcmp("-?", argv[k])) ||
-                 (0 == strncmp("-h", argv[k], 2))) {
-            printf("Scan sg device names and optionally do an INQUIRY\n\n");
-            usage();
-            return 1;
-        }
-        else if (*argv[k] == '-') {
-            printf("Unknown switch: %s\n", argv[k]);
-            usage();
-            return 1;
-        }
-        else if (*argv[k] != '-') {
-	    if (j < max_file_args) {
-		has_file_args = 1;
-		argv_index_arr[j++] = k;
-	    } else {
-	        printf("Too many command line arguments\n");
-		return 1;
-	    }
+        cp = argv[k];
+        plen = strlen(cp);
+        if (plen <= 0)
+            continue;
+        if ('-' == *cp) {
+            for (--plen, ++cp, jmp_out = 0; plen > 0; --plen, ++cp) {
+                switch (*cp) {
+                case 'a':
+                    do_numeric = 0;
+                    break;
+                case 'h':
+                case '?':
+                    printf("Scan sg device names and optionally do an "
+                           "INQUIRY\n\n");
+                    usage();
+                    return 1;
+                case 'i':
+                    do_inquiry = 1;
+                    break;
+                case 'n':
+                    do_numeric = 1;
+                    break;
+                case 'v':
+                    ++verbose;
+                    break;
+                case 'V':
+                    fprintf(stderr, "Version string: %s\n", version_str);
+                    exit(0);
+                case 'w':
+                    writeable = 1;
+                    break;
+                case 'x':
+                    do_extra = 1;
+                    break;
+                default:
+                    jmp_out = 1;
+                    break;
+                }
+                if (jmp_out)
+                    break;
+            }
+            if (plen <= 0)
+                continue;
+            if (jmp_out) {
+                fprintf(stderr, "Unrecognized option: %s\n", cp);
+                usage();
+                return 1;
+            }
+        } else {
+            if (j < max_file_args) {
+                has_file_args = 1;
+                argv_index_arr[j++] = k;
+            } else {
+                printf("Too many command line arguments\n");
+                return 1;
+            }
         }
     }
-
+    
     flags = writeable ? O_RDWR : O_RDONLY;
 
     for (k = 0, res = 0, j = 0; 
-	 (k < max_file_args)  && (has_file_args || (num_errors < MAX_ERRORS));
+         (k < max_file_args)  && (has_file_args || (num_errors < MAX_ERRORS));
          ++k, res = (sg_fd >= 0) ? close(sg_fd) : 0) {
         if (res < 0) {
             snprintf(ebuff, EBUFF_SZ, ME "Error closing %s ", fname);
             perror(ME "close error");
             return 1;
         }
-	if (has_file_args) {
-	    if (argv_index_arr[j])
-		file_namep = argv[argv_index_arr[j++]];
-	    else
-		break;
-	} else {
+        if (has_file_args) {
+            if (argv_index_arr[j])
+                file_namep = argv[argv_index_arr[j++]];
+            else
+                break;
+        } else {
             make_dev_name(fname, k, do_numeric);
-	    file_namep = fname;
-	}
+            file_namep = fname;
+        }
 
         sg_fd = open(file_namep, flags | O_NONBLOCK);
         if (sg_fd < 0) {
@@ -216,6 +244,9 @@ int main(int argc, char * argv[])
             }
             else if ((ENODEV == errno) || (ENOENT == errno) ||
                      (ENXIO == errno)) {
+                if (verbose)
+                    fprintf(stderr, "Unable to open: %s, errno=%d\n",
+                            file_namep, errno);
                 ++num_errors;
                 ++num_silent;
                 continue;
@@ -231,11 +262,11 @@ int main(int argc, char * argv[])
         }
         res = ioctl(sg_fd, SCSI_IOCTL_GET_IDLUN, &my_idlun);
         if (res < 0) {
-	    res = try_ata_identity(file_namep, sg_fd, do_inquiry);
+            res = try_ata_identity(file_namep, sg_fd, do_inquiry);
             if (res == 0)
-		continue;
-            snprintf(ebuff, EBUFF_SZ,
-                     ME "device %s failed on scsi ioctl, skip", file_namep);
+                continue;
+            snprintf(ebuff, EBUFF_SZ, ME "device %s failed on scsi+ata "
+                     "ioctl, skip", file_namep);
             perror(ebuff);
             ++num_errors;
             continue;
@@ -250,7 +281,7 @@ int main(int argc, char * argv[])
         }
         res = ioctl(sg_fd, SG_EMULATED_HOST, &emul);
         if (res < 0)
-	    emul = -1;
+            emul = -1;
         printf("%s: scsi%d channel=%d id=%d lun=%d", file_namep, host_no,
                (my_idlun.dev_id >> 16) & 0xff, my_idlun.dev_id & 0xff,
                (my_idlun.dev_id >> 8) & 0xff);
@@ -264,8 +295,8 @@ int main(int argc, char * argv[])
 
             res = ioctl(sg_fd, SG_GET_SCSI_ID, &m_id);
             if (res < 0) {
-                snprintf(ebuff, EBUFF_SZ, ME "device %s ioctls(4), skip",
-                         file_namep);
+                snprintf(ebuff, EBUFF_SZ, ME "device %s failed "
+                         "SG_GET_SCSI_IDioctl(4), skip", file_namep);
                 perror(ebuff);
                 ++num_errors;
                 continue;
@@ -277,8 +308,8 @@ int main(int argc, char * argv[])
             else
                 printf("\n");
         }
-	else
-	    printf("\n");
+        else
+            printf("\n");
         if (do_inquiry) {
             if (-1 == sg_ver3) {
                 sg_ver3 = 0;
@@ -291,7 +322,7 @@ int main(int argc, char * argv[])
         }
     }
     if ((num_errors >= MAX_ERRORS) && (num_silent < num_errors) &&
-	(! has_file_args)) {
+        (! has_file_args)) {
         printf("Stopping because there are too many error\n");
         if (eacces_err)
             printf("    root access may be required\n");
@@ -306,6 +337,8 @@ int sg3_inq(int sg_fd, unsigned char * inqBuff, int do_extra)
     int ok, err, sg_io;
 
     memset(&io_hdr, 0, sizeof(struct sg_io_hdr));
+    memset(inqBuff, 0, INQ_REPLY_LEN);
+    inqBuff[0] = 0x7f;
     io_hdr.interface_id = 'S';
     io_hdr.cmd_len = sizeof(inqCmdBlk);
     io_hdr.mx_sb_len = sizeof(sense_buffer);
@@ -322,20 +355,22 @@ int sg3_inq(int sg_fd, unsigned char * inqBuff, int do_extra)
         if ((err = scsi_inq(sg_fd, inqBuff)) < 0) {
             perror(ME "Inquiry SG_IO + SCSI_IOCTL_SEND_COMMAND ioctl error");
             return 1;
-	} else if (err) {
+        } else if (err) {
             printf(ME "SCSI_IOCTL_SEND_COMMAND ioctl error=0x%x\n", err);
             return 1;
-	}
-        sg_io = 1;
+        }
     } else {
+        sg_io = 1;
         /* now for the error processing */
         switch (sg_err_category3(&io_hdr)) {
-        case SG_ERR_CAT_CLEAN:
-        case SG_ERR_CAT_RECOVERED:
+        case SG_LIB_CAT_RECOVERED:
+            sg_chk_n_print3("Inquiry, continuing", &io_hdr, 1);
+            /* fall through */
+        case SG_LIB_CAT_CLEAN:
             break;
         default: /* won't bother decoding other categories */
             ok = 0;
-            sg_chk_n_print3("INQUIRY command error", &io_hdr);
+            sg_chk_n_print3("INQUIRY command error", &io_hdr, 1);
             break;
         }
     }
@@ -346,7 +381,7 @@ int sg3_inq(int sg_fd, unsigned char * inqBuff, int do_extra)
         printf("    %.8s  %.16s  %.4s ", p + 8, p + 16, p + 32);
         printf("[rmb=%d cmdq=%d pqual=%d pdev=0x%x] ",
                !!(p[1] & 0x80), !!(p[7] & 2), (p[0] & 0xe0) >> 5, 
-	       (p[0] & 0x1f));
+               (p[0] & 0x1f));
         if (do_extra && sg_io)
             printf("dur=%ums\n", io_hdr.duration);
         else
@@ -369,16 +404,17 @@ int scsi_inq(int sg_fd, unsigned char * inqBuff)
     unsigned char buff[512];
     struct lscsi_ioctl_command * sicp = (struct lscsi_ioctl_command *)buff;
 
+    memset(buff, 0, sizeof(buff));
     sicp->inlen = 0;
     sicp->outlen = INQ_REPLY_LEN;
     memcpy(sicp->data, inqCmdBlk, INQ_CMD_LEN);
     res = ioctl(sg_fd, SCSI_IOCTL_SEND_COMMAND, sicp);
     if (0 == res)
-	memcpy(inqBuff, sicp->data, INQ_REPLY_LEN);
+        memcpy(inqBuff, sicp->data, INQ_REPLY_LEN);
     return res;
 }
 
-/* Following code permits ATA IDENTITY commands to be performed on
+/* Following code permits ATA IDENTIFY commands to be performed on
    ATA non "Packet Interface" devices (e.g. ATA disks).
    GPL-ed code borrowed from smartmontools (smartmontools.sf.net).
    Copyright (C) 2002-4 Bruce Allen
@@ -423,7 +459,7 @@ void swapbytes(char *out, const char *in, size_t n)
         for (k = 0; k < (n - 1); k += 2) {
             out[k] = in[k + 1];
             out[k + 1] = in[k];
-	}
+        }
     }
 }
 
@@ -479,12 +515,12 @@ void printswap(char *output, char *in, unsigned int n)
         printf("%.*s   ", (int)n, "[No Information Found]\n");
 }
 
-#define ATA_IDENTITY_BUFF_SZ  sizeof(struct ata_identify_device)
+#define ATA_IDENTIFY_BUFF_SZ  sizeof(struct ata_identify_device)
+#define HDIO_DRIVE_CMD_OFFSET 4
 
 int ata_command_interface(int device, char *data)
 {
-    const int HDIO_DRIVE_CMD_OFFSET = 4;
-    unsigned char buff[ATA_IDENTITY_BUFF_SZ + HDIO_DRIVE_CMD_OFFSET];
+    unsigned char buff[ATA_IDENTIFY_BUFF_SZ + HDIO_DRIVE_CMD_OFFSET];
     int retval; 
 
     buff[0] = ATA_IDENTIFY_DEVICE;
@@ -494,7 +530,7 @@ int ata_command_interface(int device, char *data)
         return retval;
 
     /* if the command returns data, copy it back */
-    memcpy(data, buff + HDIO_DRIVE_CMD_OFFSET, ATA_IDENTITY_BUFF_SZ);
+    memcpy(data, buff + HDIO_DRIVE_CMD_OFFSET, ATA_IDENTIFY_BUFF_SZ);
     return 0;
 }
 
@@ -508,14 +544,14 @@ int try_ata_identity(const char * file_namep, int ata_fd, int do_inq)
 
     res = ata_command_interface(ata_fd, (char *)&ata_ident);
     if (res)
-	return res;
+        return res;
     printf("%s: ATA device\n", file_namep);
     if (do_inq) {
-	printf("    ");
+        printf("    ");
         printswap(model, (char *)ata_ident.model, 40);
         printswap(serial, (char *)ata_ident.serial_no, 20);
         printswap(firm, (char *)ata_ident.fw_rev, 8);
-	printf("\n");
+        printf("\n");
     }
     return res;
 }
