@@ -17,15 +17,16 @@
 #include <sys/mman.h>
 #include <sys/time.h>
 #include <linux/major.h> 
-#include <linux/fs.h> 
+#include <linux/fs.h>   /* <sys/mount.h> */
 #include "sg_include.h"
-#include "sg_err.h"
+#include "sg_lib.h"
+#include "sg_cmds.h"
 #include "llseek.h"
 
 /* A utility program for copying files. Specialised for "files" that
 *  represent devices that understand the SCSI command set.
 *
-*  Copyright (C) 1999 - 2004 D. Gilbert and P. Allworth
+*  Copyright (C) 1999 - 2005 D. Gilbert and P. Allworth
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
@@ -37,18 +38,15 @@
    This program complains if 'ibs' or 'obs' are given with a value
    that differs from 'bs' (or the default 512).
    If 'if' is not given or 'if=-' then stdin is assumed. If 'of' is
-   not given or 'of=-' then stdout assumed. Multipliers:
-     'c','C'  *1       'b','B' *512      'k' *1024      'K' *1000
-     'm' *(1024^2)     'M' *(1000^2)     'g' *(1024^3)  'G' *(1000^3)
-     't' *(1024^4)     'T' *(1000^4)
+   not given or 'of=-' then stdout assumed.
 
    A non-standard argument "bpt" (blocks per transfer) is added to control
    the maximum number of blocks in each transfer. The default value is 128.
-   For example if "bs=512" and "bpt=32" then a maximum of 32 blocks (16KB
+   For example if "bs=512" and "bpt=32" then a maximum of 32 blocks (16 KiB
    in this case) is transferred to or from the sg device in a single SCSI
    command.
 
-   This version uses memory-mapped IO (i.e. mmap() call from the user
+   This version uses memory-mapped transfers (i.e. mmap() call from the user
    space) to speed transfers. If both sides of copy are sg devices
    then only the read side will be mmap-ed, while the write side will
    use normal IO.
@@ -56,7 +54,7 @@
    This version is designed for the linux kernel 2.4 and 2.6 series.
 */
 
-static char * version_str = "1.14 20040809";
+static char * version_str = "1.21 20050909";
 
 #define DEF_BLOCK_SIZE 512
 #define DEF_BLOCKS_PER_TRANSFER 128
@@ -73,7 +71,7 @@ static char * version_str = "1.14 20040809";
 
 #define SENSE_BUFF_LEN 32       /* Arbitrary, could be larger */
 #define READ_CAP_REPLY_LEN 8
-#define RCAP16_REPLY_LEN 12
+#define RCAP16_REPLY_LEN 32
 
 #ifndef SERVICE_ACTION_IN
 #define SERVICE_ACTION_IN     0x9e
@@ -104,6 +102,7 @@ static long long in_full = 0;
 static int in_partial = 0;
 static long long out_full = 0;
 static int out_partial = 0;
+static int verbose = 0;
 
 static const char * proc_allow_dio = "/proc/scsi/sg/allow_dio";
 
@@ -174,164 +173,142 @@ int dd_filetype(const char * filename)
     return FT_OTHER;
 }
 
+static char * dd_filetype_str(int ft, char * buff)
+{
+    int off = 0;
+
+    if (FT_DEV_NULL & ft)
+        off += snprintf(buff + off, 32, "null device ");
+    if (FT_SG & ft)
+        off += snprintf(buff + off, 32, "SCSI generic (sg) device ");
+    if (FT_BLOCK & ft)
+        off += snprintf(buff + off, 32, "block device ");
+    if (FT_ST & ft)
+        off += snprintf(buff + off, 32, "SCSI tape device ");
+    if (FT_RAW & ft)
+        off += snprintf(buff + off, 32, "raw device ");
+    if (FT_OTHER & ft)
+        off += snprintf(buff + off, 32, "other (perhaps name file) ");
+    return buff;
+}
+
 void usage()
 {
     fprintf(stderr, "Usage: "
-           "sgm_dd  [if=<infile>] [skip=<n>] [of=<ofile>] [seek=<n>]\n"
-           "               [bs=<num>] [bpt=<num>] [count=<n>] [time=<n>]\n"
-           "               [cdbsz=6|10|12|16] [fua=0|1|2|3] [sync=0|1]\n"
-           "               [dio=0|1] [--version]\n"
-           " 'bs'  must be device block size (default 512)\n"
-           " 'bpt' is blocks_per_transfer (default is 128)\n"
-           " 'time' 0->no timing(def), 1->time plus calculate throughput\n"
-           " 'fua' force unit access: 0->don't(def), 1->of, 2->if, 3->of+if\n"
-           " 'sync' 0->no sync(def), 1->SYNCHRONIZE CACHE after xfer\n"
-           " 'cdbsz' size of SCSI READ or WRITE command (default is 10)\n"
-           " 'dio'  0->indirect IO on write, 1->direct IO on write\n"
-           "        (only when read side is sg device (using mmap))\n");
+           "sgm_dd  [if=<infile>] [skip=<n>] [of=<ofile>] [seek=<n>] "
+           "[bs=<num>]\n"
+           "               [count=<n>] [ibs=<n>] [obs=<n>] [bpt=<num>] "
+           "[cdbsz=6|10|12|16]\n"
+           "               [dio=0|1] [fua=0|1|2|3] [sync=0|1] [time=<n>] "
+           "[verbose=<n>]\n"
+           "               [--version]\n"
+           "  bpt     is blocks_per_transfer (default is 128)\n"
+           "  bs      must be device block size (default 512)\n"
+           "  cdbsz   size of SCSI READ or WRITE command (default is 10)\n"
+           "  dio     0->indirect IO on write, 1->direct IO on write\n"
+           "          (only when read side is sg device (using mmap))\n"
+           "  fua     force unit access: 0->don't(def), 1->of, 2->if, "
+           "3->of+if\n");
+    fprintf(stderr, 
+           "  ibs     input block size (if given must be same as 'bs')\n"
+           "  if      file or device to read from (def stdin)\n"
+           "  obs     output block size (if given must be same as 'bs')\n"
+           "  of      file or device to write to (def stdout), name '.' "
+           "translated to\n"
+           "          /dev/null\n"
+           "  seek    block position to start writing to 'of'\n"
+           "  skip    block position to start reading from 'if'\n"
+           "  sync    0->no sync(def), 1->SYNCHRONIZE CACHE after xfer\n"
+           "  time    0->no timing(def), 1->time plus calculate "
+           "throughput\n"
+           "  verbose  0->quiet(def), 1->some noise, 2->more noise, etc\n"
+           "  --version  print version information then exit\n");
 }
 
-/* Return of 0 -> success, -1 -> failure, 2 -> try again */
-int read_capacity(int sg_fd, long long * num_sect, int * sect_sz)
+/* Return of 0 -> success, SG_LIB_CAT_INVALID_OP -> invalid opcode,
+ * SG_LIB_CAT_MEDIA_CHANGED -> media changed, SG_LIB_CAT_ILLEGAL_REQ
+ * -> bad field in cdb, -1 -> other failure */
+int scsi_read_capacity(int sg_fd, long long * num_sect, int * sect_sz)
 {
-    int res;
-    unsigned char rcCmdBlk[10] = {READ_CAPACITY, 0, 0, 0, 0, 0, 0, 0, 0, 0};
-    unsigned char rcBuff[READ_CAP_REPLY_LEN];
-    unsigned char sense_b[64];
-    struct sg_io_hdr io_hdr;
+    int k, res;
+    unsigned char rcBuff[RCAP16_REPLY_LEN];
+   int verb;
 
-    memset(&io_hdr, 0, sizeof(struct sg_io_hdr));
-    io_hdr.interface_id = 'S';
-    io_hdr.cmd_len = sizeof(rcCmdBlk);
-    io_hdr.mx_sb_len = sizeof(sense_b);
-    io_hdr.dxfer_direction = SG_DXFER_FROM_DEV;
-    io_hdr.dxfer_len = sizeof(rcBuff);
-    io_hdr.dxferp = rcBuff;
-    io_hdr.cmdp = rcCmdBlk;
-    io_hdr.sbp = sense_b;
-    io_hdr.timeout = DEF_TIMEOUT;
+    verb = (verbose ? verbose - 1: 0);
+    res = sg_ll_readcap_10(sg_fd, 0, 0, rcBuff, READ_CAP_REPLY_LEN, 0,
+                           verb);
+    if (0 != res)
+        return res;
 
-    if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
-        perror("read_capacity (SG_IO) error");
-        return -1;
-    }
-    res = sg_err_category3(&io_hdr);
-    if (SG_ERR_CAT_MEDIA_CHANGED == res)
-        return 2; /* probably have another go ... */
-    else if (SG_ERR_CAT_CLEAN != res) {
-        sg_chk_n_print3("read capacity", &io_hdr);
-        return -1;
-    }
     if ((0xff == rcBuff[0]) && (0xff == rcBuff[1]) && (0xff == rcBuff[2]) &&
         (0xff == rcBuff[3])) {
-        unsigned char rcCmdBlk16[16] = {SERVICE_ACTION_IN, 
-                            SAI_READ_CAPACITY_16,
-                            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
-        unsigned char rcBuff16[RCAP16_REPLY_LEN];
-        int k;
         long long ls;
 
-        memset(&io_hdr, 0, sizeof(struct sg_io_hdr));
-        io_hdr.interface_id = 'S';
-        io_hdr.cmd_len = sizeof(rcCmdBlk16);
-        io_hdr.mx_sb_len = sizeof(sense_b);
-        io_hdr.dxfer_direction = SG_DXFER_FROM_DEV;
-        io_hdr.dxfer_len = sizeof(rcBuff16);
-        io_hdr.dxferp = rcBuff16;
-        io_hdr.cmdp = rcCmdBlk16;
-        io_hdr.sbp = sense_b;
-        io_hdr.timeout = DEF_TIMEOUT;
-
-        if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
-            perror("read_capacity_16 (SG_IO) error");
-            return -1;
-        }
-        res = sg_err_category3(&io_hdr);
-        if (SG_ERR_CAT_CLEAN != res) {
-            sg_chk_n_print3("read capacity_16", &io_hdr);
-            return -1;
-        }
+        res = sg_ll_readcap_16(sg_fd, 0, 0, rcBuff, RCAP16_REPLY_LEN, 0,
+                               verb);
+        if (0 != res)
+            return res;
         for (k = 0, ls = 0; k < 8; ++k) {
             ls <<= 8;
-            ls |= rcBuff16[k];
+            ls |= rcBuff[k];
         }
         *num_sect = ls + 1;
-        *sect_sz = (rcBuff16[8] << 24) | (rcBuff16[9] << 16) |
-                   (rcBuff16[10] << 8) | rcBuff16[11];
+        *sect_sz = (rcBuff[8] << 24) | (rcBuff[9] << 16) |
+                   (rcBuff[10] << 8) | rcBuff[11];
     } else {
         *num_sect = 1 + ((rcBuff[0] << 24) | (rcBuff[1] << 16) |
                     (rcBuff[2] << 8) | rcBuff[3]);
         *sect_sz = (rcBuff[4] << 24) | (rcBuff[5] << 16) |
                    (rcBuff[6] << 8) | rcBuff[7];
     }
+    if (verbose)
+        fprintf(stderr, "      number of blocks=%lld [0x%llx], block "
+                "size=%d\n", *num_sect, *num_sect, *sect_sz);
     return 0;
 }
 
-/* Return of 0 -> success, -1 -> failure */
+/* Return of 0 -> success, -1 -> failure. BLKGETSIZE64, BLKGETSIZE and */
+/* BLKSSZGET macros problematic (from <linux/fs.h> or <sys/mount.h>). */
 int read_blkdev_capacity(int sg_fd, long long * num_sect, int * sect_sz)
 {
-#ifdef BLKGETSIZE64
-    unsigned long long ull;
-
-    if (ioctl(sg_fd, BLKGETSIZE64, &ull) < 0) {
-
-        perror("BLKGETSIZE64 ioctl error");
-        return -1;
-    }
-    if (ioctl(sg_fd, BLKSSZGET, sect_sz) < 0) {
+#ifdef BLKSSZGET
+    if ((ioctl(sg_fd, BLKSSZGET, sect_sz) < 0) && (*sect_sz > 0)) {
         perror("BLKSSZGET ioctl error");
         return -1;
+    } else {
+ #ifdef BLKGETSIZE64
+        unsigned long long ull;
+
+        if (ioctl(sg_fd, BLKGETSIZE64, &ull) < 0) {
+
+            perror("BLKGETSIZE64 ioctl error");
+            return -1;
+        }
+        *num_sect = ((long long)ull / (long long)*sect_sz);
+        if (verbose)
+            fprintf(stderr, "      [bgs64] number of blocks=%lld [0x%llx], "
+                    "block size=%d\n", *num_sect, *num_sect, *sect_sz);
+ #else
+        unsigned long ul;
+
+        if (ioctl(sg_fd, BLKGETSIZE, &ul) < 0) {
+            perror("BLKGETSIZE ioctl error");
+            return -1;
+        }
+        *num_sect = (long long)ul;
+        if (verbose)
+            fprintf(stderr, "      [bgs] number of blocks=%lld [0x%llx], "
+                    " block size=%d\n", *num_sect, *num_sect, *sect_sz);
+ #endif
     }
-    *num_sect = ((long long)ull / (long long)*sect_sz);
+    return 0;
 #else
-    unsigned long ul;
-
-    if (ioctl(sg_fd, BLKGETSIZE, &ul) < 0) {
-        perror("BLKGETSIZE ioctl error");
-        return -1;
-    }
-    *num_sect = (long long)ul;
-    if (ioctl(sg_fd, BLKSSZGET, sect_sz) < 0) {
-        perror("BLKSSZGET ioctl error");
-        return -1;
-    }
+    if (verbose)
+        fprintf(stderr, "      BLKSSZGET+BLKGETSIZE ioctl not available\n");
+    *num_sect = 0;
+    *sect_sz = 0;
+    return -1;
 #endif
-    return 0;
-}
-
-
-/* Return of 0 -> success, -1 -> failure, 2 -> try again */
-int sync_cache(int sg_fd)
-{
-    int res;
-    unsigned char scCmdBlk [10] = {SYNCHRONIZE_CACHE, 0, 0, 0, 0, 0, 0, 
-                                   0, 0, 0};
-    unsigned char sense_b[64];
-    struct sg_io_hdr io_hdr;
-
-    memset(&io_hdr, 0, sizeof(struct sg_io_hdr));
-    io_hdr.interface_id = 'S';
-    io_hdr.cmd_len = sizeof(scCmdBlk);
-    io_hdr.mx_sb_len = sizeof(sense_b);
-    io_hdr.dxfer_direction = SG_DXFER_NONE;
-    io_hdr.dxfer_len = 0;
-    io_hdr.dxferp = NULL;
-    io_hdr.cmdp = scCmdBlk;
-    io_hdr.sbp = sense_b;
-    io_hdr.timeout = DEF_TIMEOUT;
-
-    if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
-        perror("synchronize_cache (SG_IO) error");
-        return -1;
-    }
-    res = sg_err_category3(&io_hdr);
-    if (SG_ERR_CAT_MEDIA_CHANGED == res)
-        return 2; /* probably have another go ... */
-    else if (SG_ERR_CAT_CLEAN != res) {
-        sg_chk_n_print3("synchronize cache", &io_hdr);
-        return -1;
-    }
-    return 0;
 }
 
 int sg_build_scsi_cdb(unsigned char * cdbp, int cdb_sz, unsigned int blocks,
@@ -434,7 +411,7 @@ int sg_read(int sg_fd, unsigned char * buff, int blocks, long long from_block,
     unsigned char rdCmd[MAX_SCSI_CDBSZ];
     unsigned char senseBuff[SENSE_BUFF_LEN];
     struct sg_io_hdr io_hdr;
-    int res;
+    int k;
 
     if (sg_build_scsi_cdb(rdCmd, cdbsz, blocks, from_block, 0, fua, 0)) {
         fprintf(stderr, ME "bad rd cdb build, from_block=%lld, blocks=%d\n",
@@ -455,7 +432,19 @@ int sg_read(int sg_fd, unsigned char * buff, int blocks, long long from_block,
     io_hdr.pack_id = (int)from_block;
     if (do_mmap)
         io_hdr.flags |= SG_FLAG_MMAP_IO;
+    if (verbose > 2) {
+        fprintf(stderr, "    read cdb: ");
+        for (k = 0; k < cdbsz; ++k)
+            fprintf(stderr, "%02x ", rdCmd[k]);
+        fprintf(stderr, "\n");
+    }
 
+#if 1
+    if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
+        perror(ME "SG_IO error (sg_read)");
+        return -1;
+    }
+#else
     while (((res = write(sg_fd, &io_hdr, sizeof(io_hdr))) < 0) &&
            (EINTR == errno))
         ;
@@ -473,17 +462,19 @@ int sg_read(int sg_fd, unsigned char * buff, int blocks, long long from_block,
         perror("reading (rd) on sg device, error");
         return -1;
     }
+#endif
+    if (verbose > 2)
+        fprintf(stderr, "      duration=%u ms\n", io_hdr.duration);
     switch (sg_err_category3(&io_hdr)) {
-    case SG_ERR_CAT_CLEAN:
+    case SG_LIB_CAT_CLEAN:
         break;
-    case SG_ERR_CAT_RECOVERED:
-        fprintf(stderr, "Recovered error while reading block=%lld, num=%d\n",
-               from_block, blocks);
+    case SG_LIB_CAT_RECOVERED:
+        sg_chk_n_print3("Reading, continuing", &io_hdr, verbose);
         break;
-    case SG_ERR_CAT_MEDIA_CHANGED:
+    case SG_LIB_CAT_MEDIA_CHANGED:
         return 2;
     default:
-        sg_chk_n_print3("reading", &io_hdr);
+        sg_chk_n_print3("reading", &io_hdr, verbose);
         return -1;
     }
     sum_of_resids += io_hdr.resid;
@@ -501,7 +492,7 @@ int sg_write(int sg_fd, unsigned char * buff, int blocks, long long to_block,
     unsigned char wrCmd[MAX_SCSI_CDBSZ];
     unsigned char senseBuff[SENSE_BUFF_LEN];
     struct sg_io_hdr io_hdr;
-    int res;
+    int k;
 
     if (sg_build_scsi_cdb(wrCmd, cdbsz, blocks, to_block, 1, fua, 0)) {
         fprintf(stderr, ME "bad wr cdb build, to_block=%lld, blocks=%d\n",
@@ -525,7 +516,19 @@ int sg_write(int sg_fd, unsigned char * buff, int blocks, long long to_block,
         io_hdr.flags |= SG_FLAG_MMAP_IO;
     if (diop && *diop)
         io_hdr.flags |= SG_FLAG_DIRECT_IO;
+    if (verbose > 2) {
+        fprintf(stderr, "    write cdb: ");
+        for (k = 0; k < cdbsz; ++k)
+            fprintf(stderr, "%02x ", wrCmd[k]);
+        fprintf(stderr, "\n");
+    }
 
+#if 1
+    if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
+        perror(ME "SG_IO error (sg_write)");
+        return -1;
+    }
+#else
     while (((res = write(sg_fd, &io_hdr, sizeof(io_hdr))) < 0) &&
            (EINTR == errno))
         ;
@@ -543,17 +546,19 @@ int sg_write(int sg_fd, unsigned char * buff, int blocks, long long to_block,
         perror("writing (rd) on sg device, error");
         return -1;
     }
+#endif
+    if (verbose > 2)
+        fprintf(stderr, "      duration=%u ms\n", io_hdr.duration);
     switch (sg_err_category3(&io_hdr)) {
-    case SG_ERR_CAT_CLEAN:
+    case SG_LIB_CAT_CLEAN:
         break;
-    case SG_ERR_CAT_RECOVERED:
-        fprintf(stderr, "Recovered error while writing block=%lld, num=%d\n",
-               to_block, blocks);
+    case SG_LIB_CAT_RECOVERED:
+        sg_chk_n_print3("Writing, continuing", &io_hdr, verbose);
         break;
-    case SG_ERR_CAT_MEDIA_CHANGED:
+    case SG_LIB_CAT_MEDIA_CHANGED:
         return 2;
     default:
-        sg_chk_n_print3("writing", &io_hdr);
+        sg_chk_n_print3("writing", &io_hdr, verbose);
         return -1;
     }
     if (diop && *diop &&
@@ -562,99 +567,6 @@ int sg_write(int sg_fd, unsigned char * buff, int blocks, long long to_block,
     return 0;
 }
 
-int get_num(char * buf)
-{
-    int res, num;
-    unsigned int unum;
-    char c = 'c';
-
-    if ('\0' == buf[0])
-        return -1;
-    if (('0' == buf[0]) && (('x' == buf[1]) || ('X' == buf[1]))) {
-        res = sscanf(buf + 2, "%x", &unum);
-        num = unum;
-    } else
-        res = sscanf(buf, "%d%c", &num, &c);
-    if (1 == res)
-        return num;
-    else if (2 != res)
-        return -1;
-    else {
-        switch (c) {
-        case 'c':
-        case 'C':
-            return num;
-        case 'b':
-        case 'B':
-            return num * 512;
-        case 'k':
-            return num * 1024;
-        case 'K':
-            return num * 1000;
-        case 'm':
-            return num * 1024 * 1024;
-        case 'M':
-            return num * 1000000;
-        case 'g':
-            return num * 1024 * 1024 * 1024;
-        case 'G':
-            return num * 1000000000;
-        default:
-            fprintf(stderr, "unrecognized multiplier\n");
-            return -1;
-        }
-    }
-}
-
-long long get_llnum(char * buf)
-{
-    int res;
-    long long num;
-    unsigned long long unum;
-    char c = 'c';
-
-    if ('\0' == buf[0])
-        return -1;
-    if (('0' == buf[0]) && (('x' == buf[1]) || ('X' == buf[1]))) {
-        res = sscanf(buf + 2, "%llx", &unum);
-        num = unum;
-    } else
-        res = sscanf(buf, "%lld%c", &num, &c);
-    if (1 == res)
-        return num;
-    else if (2 != res)
-        return -1LL;
-    else {
-        switch (c) {
-        case 'c':
-        case 'C':
-            return num;
-        case 'b':
-        case 'B':
-            return num * 512;
-        case 'k':
-            return num * 1024;
-        case 'K':
-            return num * 1000;
-        case 'm':
-            return num * 1024 * 1024;
-        case 'M':
-            return num * 1000000;
-        case 'g':
-            return num * 1024 * 1024 * 1024;
-        case 'G':
-            return num * 1000000000;
-        case 't':
-            return num * 1024LL * 1024LL * 1024LL * 1024LL;
-        case 'T':
-            return num * 1000000000000LL;
-        default:
-            fprintf(stderr, "unrecognized multiplier\n");
-            return -1LL;
-        }
-    }
-}
-
 #define STR_SZ 1024
 #define INOUTF_SZ 512
 #define EBUFF_SZ 512
@@ -726,31 +638,61 @@ int main(int argc, char * argv[])
                 return 1;
             } else
                 strncpy(outf, buf, INOUTF_SZ);
-        } else if (0 == strcmp(key,"ibs"))
-            ibs = get_num(buf);
-        else if (0 == strcmp(key,"obs"))
-            obs = get_num(buf);
-        else if (0 == strcmp(key,"bs"))
-            bs = get_num(buf);
-        else if (0 == strcmp(key,"bpt"))
-            bpt = get_num(buf);
-        else if (0 == strcmp(key,"skip"))
-            skip = get_llnum(buf);
-        else if (0 == strcmp(key,"seek"))
-            seek = get_llnum(buf);
-        else if (0 == strcmp(key,"count"))
-            dd_count = get_llnum(buf);
-        else if (0 == strcmp(key,"time"))
-            do_time = get_num(buf);
+        } else if (0 == strcmp(key,"ibs")) {
+            ibs = sg_get_num(buf);
+            if (-1 == ibs) {
+                fprintf(stderr, ME "bad argument to 'ibs'\n");
+                return 1;
+            }
+        } else if (0 == strcmp(key,"obs")) {
+            obs = sg_get_num(buf);
+            if (-1 == obs) {
+                fprintf(stderr, ME "bad argument to 'obs'\n");
+                return 1;
+            }
+        } else if (0 == strcmp(key,"bs")) {
+            bs = sg_get_num(buf);
+            if (-1 == bs) {
+                fprintf(stderr, ME "bad argument to 'bs'\n");
+                return 1;
+            }
+        } else if (0 == strcmp(key,"bpt")) {
+            bpt = sg_get_num(buf);
+            if (-1 == bpt) {
+                fprintf(stderr, ME "bad argument to 'bpt'\n");
+                return 1;
+            }
+        } else if (0 == strcmp(key,"skip")) {
+            skip = sg_get_llnum(buf);
+            if (-1LL == skip) {
+                fprintf(stderr, ME "bad argument to 'skip'\n");
+                return 1;
+            }
+        } else if (0 == strcmp(key,"seek")) {
+            seek = sg_get_llnum(buf);
+            if (-1LL == seek) {
+                fprintf(stderr, ME "bad argument to 'seek'\n");
+                return 1;
+            }
+        } else if (0 == strcmp(key,"count")) {
+            dd_count = sg_get_llnum(buf);
+            if (-1LL == dd_count) {
+                fprintf(stderr, ME "bad argument to 'count'\n");
+                return 1;
+            }
+        } else if (0 == strcmp(key,"time"))
+            do_time = sg_get_num(buf);
         else if (0 == strcmp(key,"cdbsz")) {
-            scsi_cdbsz_in = get_num(buf);
+            scsi_cdbsz_in = sg_get_num(buf);
             scsi_cdbsz_out = scsi_cdbsz_in;
         } else if (0 == strcmp(key,"fua"))
-            fua_mode = get_num(buf);
+            fua_mode = sg_get_num(buf);
         else if (0 == strcmp(key,"sync"))
-            do_sync = get_num(buf);
+            do_sync = sg_get_num(buf);
         else if (0 == strcmp(key,"dio"))
-            do_dio = get_num(buf);
+            do_dio = sg_get_num(buf);
+        else if (0 == strncmp(key, "verb", 4))
+            verbose = sg_get_num(buf);
         else if (0 == strncmp(key, "--vers", 6)) {
             fprintf(stderr, ME "for Linux sg version 3 driver: %s\n",
                     version_str);
@@ -792,13 +734,16 @@ int main(int argc, char * argv[])
     outfd = STDOUT_FILENO;
     if (inf[0] && ('-' != inf[0])) {
         in_type = dd_filetype(inf);
+        if (verbose)
+            fprintf(stderr, " >> Input file type: %s\n",
+                    dd_filetype_str(in_type, ebuff));
 
         if (FT_ST == in_type) {
             fprintf(stderr, ME "unable to use scsi tape device %s\n", inf);
             return 1;
         }
         else if (FT_SG == in_type) {
-            if ((infd = open(inf, O_RDWR)) < 0) {
+            if ((infd = open(inf, O_RDWR | O_NONBLOCK)) < 0) {
                 snprintf(ebuff, EBUFF_SZ, 
                          ME "could not open %s for sg reading", inf);
                 perror(ebuff);
@@ -848,19 +793,25 @@ int main(int argc, char * argv[])
                     perror(ebuff);
                     return 1;
                 }
+                if (verbose)
+                    fprintf(stderr, "  >> skip: llseek SEEK_SET, "
+                            "byte offset=0x%llx\n", offset);
             }
         }
     }
 
     if (outf[0] && ('-' != outf[0])) {
         out_type = dd_filetype(outf);
+        if (verbose)
+            fprintf(stderr, " >> Output file type: %s\n",
+                    dd_filetype_str(out_type, ebuff));
 
         if (FT_ST == out_type) {
             fprintf(stderr, ME "unable to use scsi tape device %s\n", outf);
             return 1;
         }
         else if (FT_SG == out_type) {
-            if ((outfd = open(outf, O_RDWR)) < 0) {
+            if ((outfd = open(outf, O_RDWR | O_NONBLOCK)) < 0) {
                 snprintf(ebuff, EBUFF_SZ, ME "could not open %s for "
                          "sg writing", outf);
                 perror(ebuff);
@@ -922,6 +873,9 @@ int main(int argc, char * argv[])
                     perror(ebuff);
                     return 1;
                 }
+                if (verbose)
+                    fprintf(stderr, "   >> seek: llseek SEEK_SET, "
+                            "byte offset=0x%llx\n", offset);
             }
         }
     }
@@ -933,14 +887,18 @@ int main(int argc, char * argv[])
     if (dd_count < 0) {
         in_num_sect = -1;
         if (FT_SG == in_type) {
-            res = read_capacity(infd, &in_num_sect, &in_sect_sz);
-            if (2 == res) {
+            res = scsi_read_capacity(infd, &in_num_sect, &in_sect_sz);
+            if (SG_LIB_CAT_MEDIA_CHANGED == res) {
                 fprintf(stderr, 
                         "Unit attention, media changed(in), continuing\n");
-                res = read_capacity(infd, &in_num_sect, &in_sect_sz);
+                res = scsi_read_capacity(infd, &in_num_sect, &in_sect_sz);
             }
             if (0 != res) {
-                fprintf(stderr, "Unable to read capacity on %s\n", inf);
+               if (res == SG_LIB_CAT_INVALID_OP)
+                    fprintf(stderr, "read capacity not supported on %s\n",
+                            inf);
+                else
+                    fprintf(stderr, "Unable to read capacity on %s\n", inf);
                 in_num_sect = -1;
             }
         } else if (FT_BLOCK == in_type) {
@@ -959,14 +917,18 @@ int main(int argc, char * argv[])
 
         out_num_sect = -1;
         if (FT_SG == out_type) {
-            res = read_capacity(outfd, &out_num_sect, &out_sect_sz);
-            if (2 == res) {
+            res = scsi_read_capacity(outfd, &out_num_sect, &out_sect_sz);
+            if (SG_LIB_CAT_MEDIA_CHANGED == res) {
                 fprintf(stderr, 
                         "Unit attention, media changed(out), continuing\n");
-                res = read_capacity(outfd, &out_num_sect, &out_sect_sz);
+                res = scsi_read_capacity(outfd, &out_num_sect, &out_sect_sz);
             }
             if (0 != res) {
-                fprintf(stderr, "Unable to read capacity on %s\n", outf);
+               if (res == SG_LIB_CAT_INVALID_OP)
+                    fprintf(stderr, "read capacity not supported on %s\n",
+                            outf);
+                else
+                    fprintf(stderr, "Unable to read capacity on %s\n", outf);
                 out_num_sect = -1;
             }
         } else if (FT_BLOCK == out_type) {
@@ -1070,6 +1032,11 @@ int main(int argc, char * argv[])
     }
     req_count = dd_count;
 
+    if (verbose && (dd_count > 0) && (0 == do_dio) &&
+        (FT_SG == in_type) && (FT_SG == out_type))
+        fprintf(stderr, "Since both 'if' and 'of' are sg devices, only do "
+                "mmap-ed transfers on 'if'\n");
+
     while (dd_count > 0) {
         blocks = (dd_count > blocks_per) ? blocks_per : dd_count;
         if (FT_SG == in_type) {
@@ -1094,6 +1061,9 @@ int main(int argc, char * argv[])
             while (((res = read(infd, wrkPos, blocks * bs)) < 0) &&
                    (EINTR == errno))
                 ;
+            if (verbose > 2)
+                fprintf(stderr, "read(unix): count=%d, res=%d\n",
+                        blocks * bs, res);
             if (res < 0) {
                 snprintf(ebuff, EBUFF_SZ, ME "reading, skip=%lld ", skip);
                 perror(ebuff);
@@ -1142,6 +1112,9 @@ int main(int argc, char * argv[])
             while (((res = write(outfd, wrkPos, blocks * bs)) < 0)
                    && (EINTR == errno))
                 ;
+            if (verbose > 2)
+                fprintf(stderr, "write(unix): count=%d, res=%d\n",
+                        blocks * bs, res);
             if (res < 0) {
                 snprintf(ebuff, EBUFF_SZ, ME "writing, seek=%lld ", seek);
                 perror(ebuff);
@@ -1187,11 +1160,11 @@ int main(int argc, char * argv[])
     if (do_sync) {
         if (FT_SG == out_type) {
             fprintf(stderr, ">> Synchronizing cache on %s\n", outf);
-            res = sync_cache(outfd);
+            res = sg_ll_sync_cache_10(outfd, 0, 0, 0, 0, 0, 0, 0);
             if (2 == res) {
                 fprintf(stderr,
                         "Unit attention, media changed(in), continuing\n");
-                res = sync_cache(outfd);
+                res = sg_ll_sync_cache_10(outfd, 0, 0, 0, 0, 0, 0, 0);
             }
             if (0 != res)
                 fprintf(stderr, "Unable to synchronize cache\n");
@@ -1213,7 +1186,7 @@ int main(int argc, char * argv[])
         fprintf(stderr, ">> Non-zero sum of residual counts=%d\n", 
                 sum_of_resids);
     if (num_dio_not_done)
-        fprintf(stderr, ">> dio requested but _not done %d times\n", 
+        fprintf(stderr, ">> dio requested but _not_ done %d times\n", 
                 num_dio_not_done);
     return res;
 }
