@@ -17,15 +17,16 @@
 #include <sys/sysmacros.h>
 #include <sys/time.h>
 #include <linux/major.h>
-#include <linux/fs.h>
+#include <linux/fs.h>   /* <sys/mount.h> */
 #include "sg_include.h"
-#include "sg_err.h"
+#include "sg_lib.h"
+#include "sg_cmds.h"
 #include "llseek.h"
 
 /* A utility program for copying files. Specialised for "files" that
 *  represent devices that understand the SCSI command set.
 *
-*  Copyright (C) 1999 - 2004 D. Gilbert and P. Allworth
+*  Copyright (C) 1999 - 2005 D. Gilbert and P. Allworth
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
@@ -36,14 +37,11 @@
    device. A block size ('bs') is assumed to be 512 if not given. This
    program complains if 'ibs' or 'obs' are given with some other value
    than 'bs'. If 'if' is not given or 'if=-' then stdin is assumed. If
-   'of' is not given or 'of=-' then stdout assumed.  Multipliers:
-      'c','C'  *1       'b','B' *512      'k' *1024      'K' *1000
-      'm' *(1024^2)     'M' *(1000^2)     'g' *(1024^3)  'G' *(1000^3)
-      't' *(1024^4)     'T' *(1000^4)
+   'of' is not given or 'of=-' then stdout assumed.
    
    A non-standard argument "bpt" (blocks per transfer) is added to control
    the maximum number of blocks in each transfer. The default value is 128.
-   For example if "bs=512" and "bpt=32" then a maximum of 32 blocks (16KB
+   For example if "bs=512" and "bpt=32" then a maximum of 32 blocks (16 KiB
    in this case) are transferred to or from the sg device in a single SCSI
    command.
 
@@ -51,7 +49,7 @@
 
 */
 
-static char * version_str = "5.18 20040810";
+static char * version_str = "5.24 20050806";
 
 #define DEF_BLOCK_SIZE 512
 #define DEF_BLOCKS_PER_TRANSFER 128
@@ -64,7 +62,7 @@ static char * version_str = "5.18 20040810";
 
 #define SENSE_BUFF_LEN 32       /* Arbitrary, could be larger */
 #define READ_CAP_REPLY_LEN 8
-#define RCAP16_REPLY_LEN 12
+#define RCAP16_REPLY_LEN 32
 
 #ifndef SERVICE_ACTION_IN
 #define SERVICE_ACTION_IN     0x9e
@@ -260,150 +258,75 @@ static void guarded_stop_both(Rq_coll * clp)
     guarded_stop_out(clp);
 }
 
-/* Return of 0 -> success, -1 -> failure, 2 -> try again */
-int read_capacity(int sg_fd, long long * num_sect, int * sect_sz)
+/* Return of 0 -> success, SG_LIB_CAT_INVALID_OP -> invalid opcode,
+ * SG_LIB_CAT_MEDIA_CHANGED -> media changed, SG_LIB_CAT_ILLEGAL_REQ
+ * -> bad field in cdb, -1 -> other failure */
+int scsi_read_capacity(int sg_fd, long long * num_sect, int * sect_sz)
 {
-    int res;
-    unsigned char rcCmdBlk[10] = {READ_CAPACITY, 0, 0, 0, 0, 0, 0, 0, 0, 0};
-    unsigned char rcBuff[READ_CAP_REPLY_LEN];
-    unsigned char sense_b[64];
-    struct sg_io_hdr io_hdr;
+    int k, res;
+    unsigned char rcBuff[RCAP16_REPLY_LEN];
 
-    memset(&io_hdr, 0, sizeof(struct sg_io_hdr));
-    io_hdr.interface_id = 'S';
-    io_hdr.cmd_len = sizeof(rcCmdBlk);
-    io_hdr.mx_sb_len = sizeof(sense_b);
-    io_hdr.dxfer_direction = SG_DXFER_FROM_DEV;
-    io_hdr.dxfer_len = sizeof(rcBuff);
-    io_hdr.dxferp = rcBuff;
-    io_hdr.cmdp = rcCmdBlk;
-    io_hdr.sbp = sense_b;
-    io_hdr.timeout = DEF_TIMEOUT;
+    res = sg_ll_readcap_10(sg_fd, 0, 0, rcBuff, READ_CAP_REPLY_LEN, 0, 0);
+    if (0 != res)
+        return res;
 
-    if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
-        perror("read_capacity (SG_IO) error");
-        return -1;
-    }
-    res = sg_err_category3(&io_hdr);
-    if (SG_ERR_CAT_MEDIA_CHANGED == res)
-        return 2; /* probably have another go ... */
-    else if (SG_ERR_CAT_CLEAN != res) {
-        sg_chk_n_print3("read capacity", &io_hdr);
-        return -1;
-    }
     if ((0xff == rcBuff[0]) && (0xff == rcBuff[1]) && (0xff == rcBuff[2]) &&
         (0xff == rcBuff[3])) {
-        unsigned char rcCmdBlk16[16] = {SERVICE_ACTION_IN, 
-                            SAI_READ_CAPACITY_16,
-                            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
-        unsigned char rcBuff16[RCAP16_REPLY_LEN];
-        int k;
         long long ls;
 
-        memset(&io_hdr, 0, sizeof(struct sg_io_hdr));
-        io_hdr.interface_id = 'S';
-        io_hdr.cmd_len = sizeof(rcCmdBlk16);
-        io_hdr.mx_sb_len = sizeof(sense_b);
-        io_hdr.dxfer_direction = SG_DXFER_FROM_DEV;
-        io_hdr.dxfer_len = sizeof(rcBuff16);
-        io_hdr.dxferp = rcBuff16;
-        io_hdr.cmdp = rcCmdBlk16;
-        io_hdr.sbp = sense_b;
-        io_hdr.timeout = DEF_TIMEOUT;
-
-        if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
-            perror("read_capacity_16 (SG_IO) error");
-            return -1;
-        }
-        res = sg_err_category3(&io_hdr);
-        if (SG_ERR_CAT_CLEAN != res) {
-            sg_chk_n_print3("read capacity_16", &io_hdr);
-            return -1;
-        }
+        res = sg_ll_readcap_16(sg_fd, 0, 0, rcBuff, RCAP16_REPLY_LEN, 0, 0);
+        if (0 != res)
+            return res;
         for (k = 0, ls = 0; k < 8; ++k) {
             ls <<= 8;
-            ls |= rcBuff16[k];
+            ls |= rcBuff[k];
         }
         *num_sect = ls + 1;
-        *sect_sz = (rcBuff16[8] << 24) | (rcBuff16[9] << 16) |
-                   (rcBuff16[10] << 8) | rcBuff16[11];
+        *sect_sz = (rcBuff[8] << 24) | (rcBuff[9] << 16) |
+                   (rcBuff[10] << 8) | rcBuff[11];
     } else {
         *num_sect = 1 + ((rcBuff[0] << 24) | (rcBuff[1] << 16) |
                     (rcBuff[2] << 8) | rcBuff[3]);
         *sect_sz = (rcBuff[4] << 24) | (rcBuff[5] << 16) |
                    (rcBuff[6] << 8) | rcBuff[7];
     }
-#ifdef SG_DEBUG
-    fprintf(stderr, "number of sectors=%lld, sector size=%d\n",
-            *num_sect, *sect_sz);
-#endif
     return 0;
 }
 
-/* Return of 0 -> success, -1 -> failure */
+/* Return of 0 -> success, -1 -> failure. BLKGETSIZE64, BLKGETSIZE and */
+/* BLKSSZGET macros problematic (from <linux/fs.h> or <sys/mount.h>). */
 int read_blkdev_capacity(int sg_fd, long long * num_sect, int * sect_sz)
 {
-#ifdef BLKGETSIZE64
-    unsigned long long ull;
-
-    if (ioctl(sg_fd, BLKGETSIZE64, &ull) < 0) {
-
-        perror("BLKGETSIZE64 ioctl error");
-        return -1;
-    }
-    if (ioctl(sg_fd, BLKSSZGET, sect_sz) < 0) {
+#ifdef BLKSSZGET
+    if ((ioctl(sg_fd, BLKSSZGET, sect_sz) < 0) && (*sect_sz > 0)) {
         perror("BLKSSZGET ioctl error");
         return -1;
+    } else {
+ #ifdef BLKGETSIZE64
+        unsigned long long ull;
+
+        if (ioctl(sg_fd, BLKGETSIZE64, &ull) < 0) {
+
+            perror("BLKGETSIZE64 ioctl error");
+            return -1;
+        }
+        *num_sect = ((long long)ull / (long long)*sect_sz);
+ #else
+        unsigned long ul;
+
+        if (ioctl(sg_fd, BLKGETSIZE, &ul) < 0) {
+            perror("BLKGETSIZE ioctl error");
+            return -1;
+        }
+        *num_sect = (long long)ul;
+ #endif
     }
-    *num_sect = ((long long)ull / (long long)*sect_sz);
+    return 0;
 #else
-    unsigned long ul;
-
-    if (ioctl(sg_fd, BLKGETSIZE, &ul) < 0) {
-        perror("BLKGETSIZE ioctl error");
-        return -1;
-    }
-    *num_sect = (long long)ul;
-    if (ioctl(sg_fd, BLKSSZGET, sect_sz) < 0) {
-        perror("BLKSSZGET ioctl error");
-        return -1;
-    }
+    *num_sect = 0;
+    *sect_sz = 0;
+    return -1;
 #endif
-    return 0;
-}
-
-/* Return of 0 -> success, -1 -> failure, 2 -> try again */
-int sync_cache(int sg_fd)
-{
-    int res;
-    unsigned char scCmdBlk [10] = {SYNCHRONIZE_CACHE, 0, 0, 0, 0, 0, 0, 
-                                   0, 0, 0};
-    unsigned char sense_b[64];
-    struct sg_io_hdr io_hdr;
-
-    memset(&io_hdr, 0, sizeof(struct sg_io_hdr));
-    io_hdr.interface_id = 'S';
-    io_hdr.cmd_len = sizeof(scCmdBlk);
-    io_hdr.mx_sb_len = sizeof(sense_b);
-    io_hdr.dxfer_direction = SG_DXFER_NONE;
-    io_hdr.dxfer_len = 0;
-    io_hdr.dxferp = NULL;
-    io_hdr.cmdp = scCmdBlk;
-    io_hdr.sbp = sense_b;
-    io_hdr.timeout = DEF_TIMEOUT;
-
-    if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
-        perror("synchronize_cache (SG_IO) error");
-        return -1;
-    }
-    res = sg_err_category3(&io_hdr);
-    if (SG_ERR_CAT_MEDIA_CHANGED == res)
-        return 2; /* probably have another go ... */
-    else if (SG_ERR_CAT_CLEAN != res) {
-        sg_chk_n_print3("synchronize cache", &io_hdr);
-        return -1;
-    }
-    return 0;
 }
 
 void * sig_listen_thread(void * v_clp)
@@ -931,13 +854,13 @@ int sg_finish_io(int wr, Rq_elem * rep, pthread_mutex_t * a_mutp)
     hp = &rep->io_hdr;
 
     switch (sg_err_category3(hp)) {
-        case SG_ERR_CAT_CLEAN:
+        case SG_LIB_CAT_CLEAN:
             break;
-        case SG_ERR_CAT_RECOVERED:
-            fprintf(stderr, "Recovered error on block=%d, num=%d\n",
-                    rep->blk, rep->num_blks);
+        case SG_LIB_CAT_RECOVERED:
+            sg_chk_n_print3((rep->wr ? "writing continuing":
+                                       "reading continuing"), hp, 0);
             break;
-        case SG_ERR_CAT_MEDIA_CHANGED:
+        case SG_LIB_CAT_MEDIA_CHANGED:
             return 1;
         default:
             {
@@ -947,7 +870,7 @@ int sg_finish_io(int wr, Rq_elem * rep, pthread_mutex_t * a_mutp)
                          "%s blk=%d", rep->wr ? "writing": "reading", rep->blk);
                 status = pthread_mutex_lock(a_mutp);
                 if (0 != status) err_exit(status, "lock aux_mutex");
-                sg_chk_n_print3(ebuff, hp);
+                sg_chk_n_print3(ebuff, hp, 0);
                 status = pthread_mutex_unlock(a_mutp);
                 if (0 != status) err_exit(status, "unlock aux_mutex");
                 return -1;
@@ -996,99 +919,6 @@ int sg_prepare(int fd, int bs, int bpt, int * scsi_typep)
     return 0;
 }
 
-int get_num(char * buf)
-{
-    int res, num;
-    unsigned int unum;
-    char c = 'c';
-
-    if ('\0' == buf[0])
-        return -1;
-    if (('0' == buf[0]) && (('x' == buf[1]) || ('X' == buf[1]))) {
-        res = sscanf(buf + 2, "%x", &unum);
-        num = unum;
-    } else
-        res = sscanf(buf, "%d%c", &num, &c);
-    if (1 == res)
-        return num;
-    else if (2 != res)
-        return -1;
-    else {
-        switch (c) {
-        case 'c':
-        case 'C':
-            return num;
-        case 'b':
-        case 'B':
-            return num * 512;
-        case 'k':
-            return num * 1024;
-        case 'K':
-            return num * 1000;
-        case 'm':
-            return num * 1024 * 1024;
-        case 'M':
-            return num * 1000000;
-        case 'g':
-            return num * 1024 * 1024 * 1024;
-        case 'G':
-            return num * 1000000000;
-        default:
-            fprintf(stderr, "unrecognized multiplier\n");
-            return -1;
-        }
-    }
-}
-
-long long get_llnum(char * buf)
-{
-    int res;
-    long long num;
-    unsigned long long unum;
-    char c = 'c';
-
-    if ('\0' == buf[0])
-        return -1;
-    if (('0' == buf[0]) && (('x' == buf[1]) || ('X' == buf[1]))) {
-        res = sscanf(buf + 2, "%llx", &unum);
-        num = unum;
-    } else
-        res = sscanf(buf, "%lld%c", &num, &c);
-    if (1 == res)
-        return num;
-    else if (2 != res)
-        return -1LL;
-    else {
-        switch (c) {
-        case 'c':
-        case 'C':
-            return num;
-        case 'b':
-        case 'B':
-            return num * 512;
-        case 'k':
-            return num * 1024;
-        case 'K':
-            return num * 1000;
-        case 'm':
-            return num * 1024 * 1024;
-        case 'M':
-            return num * 1000000;
-        case 'g':
-            return num * 1024 * 1024 * 1024;
-        case 'G':
-            return num * 1000000000;
-        case 't':
-            return num * 1024LL * 1024LL * 1024LL * 1024LL;
-        case 'T':
-            return num * 1000000000000LL;
-        default:
-            fprintf(stderr, "unrecognized multiplier\n");
-            return -1LL;
-        }
-    }
-}
-
 #define STR_SZ 1024
 #define INOUTF_SZ 512
 
@@ -1155,37 +985,65 @@ int main(int argc, char * argv[])
                 return 1;
             } else
                 strncpy(outf, buf, INOUTF_SZ);
-        } else if (0 == strcmp(key,"ibs"))
-            ibs = get_num(buf);
-        else if (0 == strcmp(key,"obs"))
-            obs = get_num(buf);
-        else if (0 == strcmp(key,"bs"))
-            rcoll.bs = get_num(buf);
-        else if (0 == strcmp(key,"bpt"))
-            rcoll.bpt = get_num(buf);
-        else if (0 == strcmp(key,"skip"))
-            skip = get_llnum(buf);
-        else if (0 == strcmp(key,"seek"))
-            seek = get_llnum(buf);
-        else if (0 == strcmp(key,"count"))
-            count = get_llnum(buf);
-        else if (0 == strcmp(key,"dio"))
-            rcoll.dio = get_num(buf);
+        } else if (0 == strcmp(key,"ibs")) {
+            ibs = sg_get_num(buf);
+            if (-1 == ibs) {
+                fprintf(stderr, ME "bad argument to 'ibs'\n");
+                return 1;
+            }
+        } else if (0 == strcmp(key,"obs")) {
+            obs = sg_get_num(buf);
+            if (-1 == obs) {
+                fprintf(stderr, ME "bad argument to 'obs'\n");
+                return 1;
+            }
+        } else if (0 == strcmp(key,"bs")) {
+            rcoll.bs = sg_get_num(buf);
+            if (-1 == rcoll.bs) {
+                fprintf(stderr, ME "bad argument to 'bs'\n");
+                return 1;
+            }
+        } else if (0 == strcmp(key,"bpt")) {
+            rcoll.bpt = sg_get_num(buf);
+            if (-1 == rcoll.bpt) {
+                fprintf(stderr, ME "bad argument to 'bpt'\n");
+                return 1;
+            }
+        } else if (0 == strcmp(key,"skip")) {
+            skip = sg_get_llnum(buf);
+            if (-1LL == skip) {
+                fprintf(stderr, ME "bad argument to 'skip'\n");
+                return 1;
+            }
+        } else if (0 == strcmp(key,"seek")) {
+            seek = sg_get_llnum(buf);
+            if (-1LL == seek) {
+                fprintf(stderr, ME "bad argument to 'seek'\n");
+                return 1;
+            }
+        } else if (0 == strcmp(key,"count")) {
+            count = sg_get_llnum(buf);
+            if (-1LL == count) {
+                fprintf(stderr, ME "bad argument to 'count'\n");
+                return 1;
+            }
+        } else if (0 == strcmp(key,"dio"))
+            rcoll.dio = sg_get_num(buf);
         else if (0 == strcmp(key,"thr"))
-            num_threads = get_num(buf);
+            num_threads = sg_get_num(buf);
         else if (0 == strcmp(key,"coe"))
-            rcoll.coe = get_num(buf);
+            rcoll.coe = sg_get_num(buf);
         else if (0 == strcmp(key,"time"))
-            do_time = get_num(buf);
+            do_time = sg_get_num(buf);
         else if (0 == strcmp(key,"cdbsz")) {
-            rcoll.cdbsz_in = get_num(buf);
+            rcoll.cdbsz_in = sg_get_num(buf);
             rcoll.cdbsz_out = rcoll.cdbsz_in;
         } else if (0 == strcmp(key,"fua"))
-            rcoll.fua_mode = get_num(buf);
+            rcoll.fua_mode = sg_get_num(buf);
         else if (0 == strcmp(key,"sync"))
-            do_sync = get_num(buf);
+            do_sync = sg_get_num(buf);
         else if (0 == strncmp(key,"deb", 3))
-            rcoll.debug = get_num(buf);
+            rcoll.debug = sg_get_num(buf);
         else if (0 == strncmp(key, "--vers", 6)) {
             fprintf(stderr, ME "for sg version 3 driver: %s\n", 
                     version_str);
@@ -1321,14 +1179,19 @@ int main(int argc, char * argv[])
     if (count < 0) {
         in_num_sect = -1;
         if (FT_SG == rcoll.in_type) {
-            res = read_capacity(rcoll.infd, &in_num_sect, &in_sect_sz);
+            res = scsi_read_capacity(rcoll.infd, &in_num_sect, &in_sect_sz);
             if (2 == res) {
                 fprintf(stderr, 
                         "Unit attention, media changed(in), continuing\n");
-                res = read_capacity(rcoll.infd, &in_num_sect, &in_sect_sz);
+                res = scsi_read_capacity(rcoll.infd, &in_num_sect,
+                                         &in_sect_sz);
             }
             if (0 != res) {
-                fprintf(stderr, "Unable to read capacity on %s\n", inf);
+               if (res == SG_LIB_CAT_INVALID_OP)
+                    fprintf(stderr, "read capacity not supported on %s\n",
+                            inf);
+                else
+                    fprintf(stderr, "Unable to read capacity on %s\n", inf);
                 in_num_sect = -1;
             }
         } else if (FT_BLOCK == rcoll.in_type) {
@@ -1348,14 +1211,19 @@ int main(int argc, char * argv[])
 
         out_num_sect = -1;
         if (FT_SG == rcoll.out_type) {
-            res = read_capacity(rcoll.outfd, &out_num_sect, &out_sect_sz);
+            res = scsi_read_capacity(rcoll.outfd, &out_num_sect, &out_sect_sz);
             if (2 == res) {
                 fprintf(stderr,         
                         "Unit attention, media changed(out), continuing\n");
-                res = read_capacity(rcoll.outfd, &out_num_sect, &out_sect_sz);
+                res = scsi_read_capacity(rcoll.outfd, &out_num_sect,
+                                         &out_sect_sz);
             }
             if (0 != res) {
-                fprintf(stderr, "Unable to read capacity on %s\n", outf);
+               if (res == SG_LIB_CAT_INVALID_OP)
+                    fprintf(stderr, "read capacity not supported on %s\n",
+                            outf);
+                else
+                    fprintf(stderr, "Unable to read capacity on %s\n", outf);
                 out_num_sect = -1;
             }
         } else if (FT_BLOCK == rcoll.out_type) {
@@ -1496,11 +1364,11 @@ int main(int argc, char * argv[])
     if (do_sync) {
         if (FT_SG == rcoll.out_type) {
             fprintf(stderr, ">> Synchronizing cache on %s\n", outf);
-            res = sync_cache(rcoll.outfd);
+            res = sg_ll_sync_cache_10(rcoll.outfd, 0, 0, 0, 0, 0, 0, 0);
             if (2 == res) {
                 fprintf(stderr,
                         "Unit attention, media changed(in), continuing\n");
-                res = sync_cache(rcoll.outfd);
+                res = sg_ll_sync_cache_10(rcoll.outfd, 0, 0, 0, 0, 0, 0, 0);
             }
             if (0 != res)
                 fprintf(stderr, "Unable to synchronize cache\n");
